searchNodes=[{"doc":"Money implements a set of functions to store, retrieve, convert and perform arithmetic on a Money.t/0 type that is composed of a currency code and a decimal currency amount. Money is very opinionated in the interests of serving as a dependable library that can underpin accounting and financial applications. This opinion expressed by ensuring that: Money must always have both a amount and a currency code. The currency code must always be valid. Money arithmetic can only be performed when both operands are of the same currency. Money amounts are represented as a Decimal . Money is serialised to the database as a custom Postgres composite type that includes both the amount and the currency. Therefore for Ecto serialization Postgres is assumed as the data store. Serialization is entirely optional and Ecto is not a package dependency. All arithmetic functions work in fixed point decimal. No rounding occurs automatically (unless expressly called out for a function). Explicit rounding obeys the rounding rules for a given currency. The rounding rules are defined by the Unicode consortium in its CLDR repository as implemented by the hex package ex_cldr . These rules define the number of fractional digits for a currency and the rounding increment where appropriate.","ref":"Money.html","title":"Money","type":"module"},{"doc":"The absolute value of a Money amount. Returns a Money.t/0 type with a positive sign for the amount. Arguments money is any valid Money.t/0 type returned by Money.new/2 Returns a Money.t/0 Example iex&gt; m = Money . new ( &quot;USD&quot; , - 100 ) iex&gt; Money . abs ( m ) Money . new ( :USD , &quot;100&quot; )","ref":"Money.html#abs/1","title":"Money.abs/1","type":"function"},{"doc":"Add two Money values. Arguments money_1 and money_2 are any valid Money.t/0 types returned by Money.new/2 Returns {:ok, money} or {:error, reason} Example iex&gt; Money . add Money . new ( :USD , 200 ) , Money . new ( :USD , 100 ) { :ok , Money . new ( :USD , 300 ) } iex&gt; Money . add Money . new ( :USD , 200 ) , Money . new ( :AUD , 100 ) { :error , { ArgumentError , &quot;Cannot add monies with different currencies. &quot; &lt;&gt; &quot;Received :USD and :AUD.&quot; } }","ref":"Money.html#add/2","title":"Money.add/2","type":"function"},{"doc":"Add two Money values and raise on error. Arguments money_1 and money_2 are any valid Money.t/0 types returned by Money.new/2 Returns a Money.t/0 struct or raises an exception Examples iex&gt; Money . add! Money . new ( :USD , 200 ) , Money . new ( :USD , 100 ) Money . new ( :USD , &quot;300&quot; ) Money . add! Money . new ( :USD , 200 ) , Money . new ( :CAD , 500 ) ** (ArgumentError) Cannot add two %Money{} with different currencies. Received :USD and :CAD.","ref":"Money.html#add!/2","title":"Money.add!/2","type":"function"},{"doc":"Compares two Money values numerically. If the first number is greater than the second #Integer&lt;1&gt; is returned, if less than Integer&lt;-1&gt; is returned. Otherwise, if both numbers are equal Integer&lt;0&gt; is returned. Arguments money_1 and money_2 are any valid Money.t/0 types returned by Money.new/2 Returns -1 | 0 | 1 or {:error, {module(), String.t}} Examples iex&gt; Money . cmp Money . new ( :USD , 200 ) , Money . new ( :USD , 100 ) 1 iex&gt; Money . cmp Money . new ( :USD , 200 ) , Money . new ( :USD , 200 ) 0 iex&gt; Money . cmp Money . new ( :USD , 200 ) , Money . new ( :USD , 500 ) - 1 iex&gt; Money . cmp Money . new ( :USD , 200 ) , Money . new ( :CAD , 500 ) { :error , { ArgumentError , &quot;Cannot compare monies with different currencies. Received :USD and :CAD.&quot; } }","ref":"Money.html#cmp/2","title":"Money.cmp/2","type":"function"},{"doc":"Compares two Money values numerically and raises on error. Arguments money_1 and money_2 are any valid Money.t/0 types returned by Money.new/2 Returns -1 | 0 | 1 or raises an exception Examples Money . cmp! Money . new ( :USD , 200 ) , Money . new ( :CAD , 500 ) ** (ArgumentError) Cannot compare monies with different currencies. Received :USD and :CAD.","ref":"Money.html#cmp!/2","title":"Money.cmp!/2","type":"function"},{"doc":"Compares two Money values numerically. If the first number is greater than the second :gt is returned, if less than :lt is returned, if both numbers are equal :eq is returned. Arguments money_1 and money_2 are any valid Money.t/0 types returned by Money.new/2 Returns :gt | :eq | :lt or {:error, {module(), String.t}} Examples iex&gt; Money . compare Money . new ( :USD , 200 ) , Money . new ( :USD , 100 ) :gt iex&gt; Money . compare Money . new ( :USD , 200 ) , Money . new ( :USD , 200 ) :eq iex&gt; Money . compare Money . new ( :USD , 200 ) , Money . new ( :USD , 500 ) :lt iex&gt; Money . compare Money . new ( :USD , 200 ) , Money . new ( :CAD , 500 ) { :error , { ArgumentError , &quot;Cannot compare monies with different currencies. Received :USD and :CAD.&quot; } }","ref":"Money.html#compare/2","title":"Money.compare/2","type":"function"},{"doc":"Compares two Money values numerically and raises on error. Arguments money_1 and money_2 are any valid Money.t/0 types returned by Money.new/2 Returns :gt | :eq | :lt or raises an exception Examples Money . compare! Money . new ( :USD , 200 ) , Money . new ( :CAD , 500 ) ** (ArgumentError) Cannot compare monies with different currencies. Received :USD and :CAD.","ref":"Money.html#compare!/2","title":"Money.compare!/2","type":"function"},{"doc":"Returns the effective cross-rate to convert from one currency to another. Arguments from is any Money.t/0 struct returned by Cldr.Currency.new/2 or a valid currency code to_currency is a valid currency code into which the money is converted rates is a Map of currency rates where the map key is an upcased atom or string and the value is a Decimal conversion factor. The default is the latest available exchange rates returned from Money.ExchangeRates.latest_rates() Examples Money . cross_rate ( Money . new ( :USD , 100 ) , :AUD , %{ USD : Decimal . new ( 1 ) , AUD : Decimal . new ( &quot;0.7345&quot; ) } ) { :ok , Decimal . new ( &quot;0.7345&quot; ) } Money . cross_rate Money . new ( :USD , 100 ) , :ZZZ , %{ USD : Decimal . new ( 1 ) , AUD : Decimal . new ( 0.7345 ) } ** (Cldr.UnknownCurrencyError) Currency :ZZZ is not known","ref":"Money.html#cross_rate/3","title":"Money.cross_rate/3","type":"function"},{"doc":"Returns the effective cross-rate to convert from one currency to another. Arguments from is any Money.t/0 struct returned by Cldr.Currency.new/2 or a valid currency code to_currency is a valid currency code into which the money is converted rates is a Map of currency rates where the map key is an upcased atom or string and the value is a Decimal conversion factor. The default is the latest available exchange rates returned from Money.ExchangeRates.latest_rates() Examples iex&gt; Money . cross_rate! ( Money . new ( :USD , 100 ) , :AUD , %{ USD : Decimal . new ( 1 ) , AUD : Decimal . new ( &quot;0.7345&quot; ) } ) Decimal . new ( &quot;0.7345&quot; ) iex&gt; Money . cross_rate! ( :USD , :AUD , %{ USD : Decimal . new ( 1 ) , AUD : Decimal . new ( &quot;0.7345&quot; ) } ) Decimal . new ( &quot;0.7345&quot; ) Money . cross_rate Money . new ( :USD , 100 ) , :ZZZ , %{ USD : Decimal . new ( 1 ) , AUD : Decimal . new ( &quot;0.7345&quot; ) } ** (Cldr.UnknownCurrencyError) Currency :ZZZ is not known","ref":"Money.html#cross_rate!/3","title":"Money.cross_rate!/3","type":"function"},{"doc":"Returns the default ex_cldr backend configured for Money , if any. If no default backing is configured, an exception is raised.","ref":"Money.html#default_backend/0","title":"Money.default_backend/0","type":"function"},{"doc":"Divide a Money value by a number. Arguments money is any valid Money.t/0 types returned by Money.new/2 number is an integer, float or Decimal.t Note that dividing one %Money{} by another is not supported. Returns {:ok, money} or {:error, reason} Example iex&gt; Money . div Money . new ( :USD , 200 ) , 2 { :ok , Money . new ( :USD , 100 ) } iex&gt; Money . div ( Money . new ( :USD , 200 ) , &quot;xx&quot; ) { :error , { ArgumentError , &quot;Cannot divide money by \\&quot; xx \\&quot; &quot; } }","ref":"Money.html#div/2","title":"Money.div/2","type":"function"},{"doc":"Divide a Money value by a number and raise on error. Arguments money is any valid Money.t/0 types returned by Money.new/2 number is an integer, float or Decimal.t Returns a Money.t/0 struct or raises an exception Examples iex&gt; Money . div! ( Money . new ( :USD , 200 ) , 2 ) Money . new ( :USD , &quot;100&quot; ) iex&gt; Money . div! ( Money . new ( :USD , 200 ) , &quot;xx&quot; ) ** (ArgumentError) Cannot divide money by &quot;xx&quot;","ref":"Money.html#div!/2","title":"Money.div!/2","type":"function"},{"doc":"Returns a boolean indicating if two Money values are equal Arguments money_1 and money_2 are any valid Money.t/0 types returned by Money.new/2 Returns true or false Example iex&gt; Money . equal? ( Money . new ( :USD , 200 ) , Money . new ( :USD , 200 ) ) true iex&gt; Money . equal? ( Money . new ( :USD , 200 ) , Money . new ( :USD , 100 ) ) false","ref":"Money.html#equal?/2","title":"Money.equal?/2","type":"function"},{"doc":"Returns a %Money{} struct from a currency code and a float amount, or an error tuple of the form {:error, {exception, message}} . Floats are fraught with danger in computer arithmetic due to the unexpected loss of precision during rounding. The IEEE754 standard indicates that a number with a precision of 16 digits should round-trip convert without loss of fidelity. This function supports numbers with a precision up to 15 digits and will error if the provided amount is outside that range. Note that Money cannot detect lack of precision or rounding errors introduced upstream. This function therefore should be used with great care and its use should be considered potentially harmful. Arguments currency_code is an ISO4217 binary or atom currency code or an ISO 24165 token identifier or shortname. amount is a float options is a keyword list of options passed to Money.new/3 . The default is [] . Examples iex&gt; Money . from_float 1.23456 , :USD Money . new ( :USD , &quot;1.23456&quot; ) iex&gt; Money . from_float 1.234567890987656 , :USD { :error , { Money.InvalidAmountError , &quot;The precision of the float 1.234567890987656 is &quot; &lt;&gt; &quot;greater than 15 which could lead to unexpected results. &quot; &lt;&gt; &quot;Reduce the precision or call Money.new/2 with a Decimal or String amount&quot; } }","ref":"Money.html#from_float/3","title":"Money.from_float/3","type":"function"},{"doc":"Returns a %Money{} struct from a currency code and a float amount, or raises an exception if the currency code is invalid. See Money.from_float/2 for further information. Note that Money cannot detect lack of precision or rounding errors introduced upstream. This function therefore should be used with great care and its use should be considered potentially harmful. Arguments currency_code is an ISO4217 binary or atom currency code or an ISO 24165 token identifier or shortname. amount is a float options is a keyword list of options passed to Money.new/3 . The default is [] . Examples iex&gt; Money . from_float! ( :USD , 1.234 ) Money . new ( :USD , &quot;1.234&quot; ) Money . from_float! ( :USD , 1.234567890987654 ) #=&gt; ** (Money.InvalidAmountError) The precision of the float 1.234567890987654 is greater than 15 which could lead to unexpected results. Reduce the precision or call Money.new/2 with a Decimal or String amount ( ex_money ) lib / money . ex : 293 : Money . from_float! / 2","ref":"Money.html#from_float!/3","title":"Money.from_float!/3","type":"function"},{"doc":"Convert an integer representation of money into a Money struct. Arguments integer is an integer representation of a money amount including any decimal digits. ie. 20000 would be interpreted to mean $200.00 if the currency is :USD and no :fractional_digits option was provided. currency is the currency code for the integer . The assumed decimal precision is derived from the currency code if no fractional_digits option is specified. options is a keyword list of options. Options :fractional_digits which determines the currency precision implied by the integer . The valid options are :cash , :accounting , :iso or a non-negative integer. The default is :iso which uses the ISO 4217 definition of currency digits. All other options are passed to Money.new/3 . Returns A Money.t/0 struct or {:error, {exception, message}} Notes Some currencies, like the Iraqi Dinar have a difference in the decimal digits defined by CLDR versus those defined by ISO 4217 . CLDR defines the decimal digits for IQD as 0 whereas ISO 4217 defines 3 decimal digits. Since converting an integer to a money amount is very sensitive to the number of fractional digits specified it is important to be very clear about the precision of the data used with this function and care taken in specifying the :fractional_digits parameter. Examples iex&gt; Money . from_integer ( 20000 , :USD ) Money . new ( :USD , &quot;200.00&quot; ) iex&gt; Money . from_integer ( 200 , :JPY ) Money . new ( :JPY , &quot;200&quot; ) iex&gt; Money . from_integer ( 20012 , :USD ) Money . new ( :USD , &quot;200.12&quot; ) iex&gt; Money . from_integer ( 20012 , :USD , fractional_digits : 3 ) Money . new ( :USD , &quot;20.012&quot; ) iex&gt; Money . from_integer ( 20012 , :IQD ) Money . new ( :IQD , &quot;20.012&quot; )","ref":"Money.html#from_integer/3","title":"Money.from_integer/3","type":"function"},{"doc":"","ref":"Money.html#get_env/3","title":"Money.get_env/3","type":"function"},{"doc":"See Cldr.known_currencies/0 .","ref":"Money.html#known_currencies/0","title":"Money.known_currencies/0","type":"function"},{"doc":"See Money.Currency.known_current_currencies/0 .","ref":"Money.html#known_current_currencies/0","title":"Money.known_current_currencies/0","type":"function"},{"doc":"See Money.Currency.known_historic_currencies/0 .","ref":"Money.html#known_historic_currencies/0","title":"Money.known_historic_currencies/0","type":"function"},{"doc":"See Money.Currency.known_tender_currencies/0 .","ref":"Money.html#known_tender_currencies/0","title":"Money.known_tender_currencies/0","type":"function"},{"doc":"Localizes a Money by converting it to the currency of the specified locale. Arguments money is any Money.t/0 struct returned by Cldr.Currency.new/2 . options is a keyword list of options. Options :locale is any valid locale returned by Cldr.known_locale_names/1 or a Cldr.LanguageTag struct returned by Cldr.Locale.new!/2 The default is &lt;backend&gt;.get_locale() :backend is any module() that includes use Cldr and therefore is a Cldr backend module(). The default is Money.default_backend() Returns {:ok, localized_money} or {:error, {exception, reason}}","ref":"Money.html#localize/2","title":"Money.localize/2","type":"function"},{"doc":"Multiply a Money value by a number. Arguments money is any valid Money.t/0 type returned by Money.new/2 number is an integer, float or Decimal.t Note that multipling one %Money{} by another is not supported. Returns {:ok, money} or {:error, reason} Example iex&gt; Money . mult ( Money . new ( :USD , 200 ) , 2 ) { :ok , Money . new ( :USD , 400 ) } iex&gt; Money . mult ( Money . new ( :USD , 200 ) , &quot;xx&quot; ) { :error , { ArgumentError , &quot;Cannot multiply money by \\&quot; xx \\&quot; &quot; } }","ref":"Money.html#mult/2","title":"Money.mult/2","type":"function"},{"doc":"Multiply a Money value by a number and raise on error. Arguments money is any valid Money.t/0 types returned by Money.new/2 number is an integer, float or Decimal.t Returns a Money.t/0 or raises an exception Examples iex&gt; Money . mult! ( Money . new ( :USD , 200 ) , 2 ) Money . new ( :USD , &quot;400&quot; ) Money . mult! ( Money . new ( :USD , 200 ) , :invalid ) ** (ArgumentError) Cannot multiply money by :invalid","ref":"Money.html#mult!/2","title":"Money.mult!/2","type":"function"},{"doc":"Returns a boolean indicating if Money.t/0 has a negative value. Arguments money is any valid Money.t/0 type returned by Money.new/2 Example iex&gt; Money . negative? ( Money . new ( :USD , - 1 ) ) true iex&gt; Money . negative? ( Money . new ( :USD , 0 ) ) false iex&gt; Money . negative? ( Money . new ( :USD , 1 ) ) false","ref":"Money.html#negative?/1","title":"Money.negative?/1","type":"function"},{"doc":"Returns a %Money{} struct from a currency code and a currency amount or an error tuple of the form {:error, {exception, message}} . Arguments currency_code is an ISO4217 binary or atom currency code or an ISO 24165 token identifier or shortname. amount is an integer, string or Decimal money amount. options is a keyword list of options. Options :locale is any known locale. The locale is used to normalize any binary (String) amounts to a form that can be consumed by Decimal.new/1 . This consists of removing any localised grouping characters and replacing the localised decimal separator with a &quot;.&quot;. The default is Cldr.get_locale/0 . :backend is any module() that includes use Cldr and therefore is a Cldr backend module(). The default is Money.default_backend/0 . Any other options are considered as formatting options to be applied by default when calling Money.to_string/2 . Note that the currency_code and amount arguments can be supplied in either order, Examples iex&gt; Money . new ( :USD , 100 ) Money . new ( :USD , &quot;100&quot; ) iex&gt; Money . new ( 100 , :USD ) Money . new ( :USD , &quot;100&quot; ) iex&gt; Money . new ( &quot;USD&quot; , 100 ) Money . new ( :USD , &quot;100&quot; ) iex&gt; Money . new ( &quot;thb&quot; , 500 ) Money . new ( :THB , &quot;500&quot; ) iex&gt; Money . new ( &quot;EUR&quot; , Decimal . new ( 100 ) ) Money . new ( :EUR , &quot;100&quot; ) iex&gt; Money . new ( :EUR , &quot;100.30&quot; ) Money . new ( :EUR , &quot;100.30&quot; ) iex&gt; Money . new ( :EUR , &quot;100.30&quot; , fractional_digits : 4 ) Money . new ( :EUR , &quot;100.30&quot; , fractional_digits : 4 ) iex&gt; Money . new ( :XYZZ , 100 ) { :error , { Money.UnknownCurrencyError , &quot;The currency :XYZZ is invalid&quot; } } iex&gt; Money . new ( &quot;1.000,99&quot; , :EUR , locale : &quot;de&quot; ) Money . new ( :EUR , &quot;1000.99&quot; ) iex&gt; Money . new 123.445 , :USD { :error , { Money.InvalidAmountError , &quot;Float amounts are not supported in new/2 due to potenial &quot; &lt;&gt; &quot;rounding and precision issues. If absolutely required, &quot; &lt;&gt; &quot;use Money.from_float/2&quot; } }","ref":"Money.html#new/3","title":"Money.new/3","type":"function"},{"doc":"Returns a %Money{} struct from a currency code and a currency amount. Raises an exception if the current code is invalid. Arguments currency_code is an ISO4217 binary or atom currency code or an ISO 24165 token identifier or shortname. amount is an integer, float or Decimal Examples Money . new! ( :XYZZ , 100 ) ** (Money.UnknownCurrencyError) Currency :XYZZ is not known ( ex_money ) lib / money . ex : 177 : Money . new! / 2","ref":"Money.html#new!/3","title":"Money.new!/3","type":"function"},{"doc":"Normalizes the underlying decimal amount in a given Money.t() This will normalize the coefficient and exponent of the decimal amount in a standard way that may aid in native comparison of %Money.t() items. Example iex&gt; x = % Money { currency : :USD , amount : % Decimal { sign : 1 , coef : 42 , exp : 0 } } Money . new ( :USD , &quot;42&quot; ) iex&gt; y = % Money { currency : :USD , amount : % Decimal { sign : 1 , coef : 4200000000 , exp : - 8 } } Money . new ( :USD , &quot;42.00000000&quot; ) iex&gt; x == y false iex&gt; y = Money . normalize ( x ) Money . new ( :USD , &quot;42&quot; ) iex&gt; x == y true","ref":"Money.html#normalize/1","title":"Money.normalize/1","type":"function"},{"doc":"Parse a string and return a Money.t/0 or an error. The string to be parsed is required to have a currency code and an amount. The currency code may be placed before the amount or after, but not both. Parsing is strict. Additional text surrounding the currency code and amount will cause the parse to fail. Arguments string is a string to be parsed options is a keyword list of options that is passed to Money.new/3 with the exception of the options listed below Options :backend is any module() that includes use Cldr and therefore is a Cldr backend module(). The default is Money.default_backend() :locale is any valid locale returned by Cldr.known_locale_names/1 or a Cldr.LanguageTag struct returned by Cldr.Locale.new!/2 The default is &lt;backend&gt;.get_locale() :only is an atom or list of atoms representing the currencies or currency types to be considered for a match. The equates to a list of acceptable currencies for parsing. See the notes below for currency types. :except is an atom or list of atoms representing the currencies or currency types to be not considered for a match. This equates to a list of unacceptable currencies for parsing. See the notes below for currency types. :fuzzy is a float greater than 0.0 and less than or equal to 1.0 which is used as input to the String.jaro_distance/2 to determine is the provided currency string is close enough to a known currency string for it to identify definitively a currency code. It is recommended to use numbers greater than 0.8 in order to reduce false positives. :default_currency is any valid currency code or false that will used if no currency code, symbol or description is indentified in the parsed string. The default is nil which means that the default currency associated with the :locale option will be used. If false then the currency assocated with the :locale option will not be used and an error will be returned if there is no currency in the string being parsed. Returns a Money.t/0 if parsing is successful or {:error, {exception, reason}} if an error is detected. Notes The :only and :except options accept a list of currency codes and/or currency types. The following types are recognised. If both :only and :except are specified, the :except entries take priority - that means any entries in :except are removed from the :only entries. :all , the default, considers all currencies :current considers those currencies that have a :to date of nil and which also is a known ISO4217 currency :historic is the opposite of :current :tender considers currencies that are legal tender :unannotated considers currencies that don't have &quot;(some string)&quot; in their names. These are usually financial instruments. Examples iex&gt; Money . parse ( &quot;USD 100&quot; ) Money . new ( :USD , &quot;100&quot; ) iex&gt; Money . parse &quot;USD 100,00&quot; , locale : &quot;de&quot; Money . new ( :USD , &quot;100.00&quot; ) iex&gt; Money . parse ( &quot;100 USD&quot; ) Money . new ( :USD , &quot;100&quot; ) iex&gt; Money . parse ( &quot;100 eurosports&quot; , fuzzy : 0.8 ) Money . new ( :EUR , &quot;100&quot; ) iex&gt; Money . parse ( &quot;100&quot; , default_currency : :EUR ) Money . new ( :EUR , &quot;100&quot; ) iex&gt; Money . parse ( &quot;100 eurosports&quot; , fuzzy : 0.9 ) { :error , { Money.UnknownCurrencyError , &quot;The currency \\&quot; eurosports \\&quot; is unknown or not supported&quot; } } iex&gt; Money . parse ( &quot;100 afghan afghanis&quot; ) Money . new ( :AFN , &quot;100&quot; ) iex&gt; Money . parse ( &quot;100&quot; , default_currency : false ) { :error , { Money.Invalid , &quot;A currency code, symbol or description must be specified but was not found in \\&quot; 100 \\&quot; &quot; } } iex&gt; Money . parse ( &quot;USD 100 with trailing text&quot; ) { :error , { Money.ParseError , &quot;Could not parse \\&quot; USD 100 with trailing text \\&quot; .&quot; } }","ref":"Money.html#parse/2","title":"Money.parse/2","type":"function"},{"doc":"Returns a boolean indicating if Money.t/0 has a positive value. Arguments money is any valid Money.t/0 type returned by Money.new/2 Example iex&gt; Money . positive? ( Money . new ( :USD , 1 ) ) true iex&gt; Money . positive? ( Money . new ( :USD , 0 ) ) false iex&gt; Money . positive? ( Money . new ( :USD , - 1 ) ) false","ref":"Money.html#positive?/1","title":"Money.positive?/1","type":"function"},{"doc":"Add format options to a Money.t/0 . Arguments money is any valid Money.t/0 type returned by Money.new/2 options is a keyword list of options. These options are used when calling Money.to_string/2 . The default is []","ref":"Money.html#put_format_options/2","title":"Money.put_format_options/2","type":"function"},{"doc":"Set the fractional part of a Money . Arguments money is any Money.t/0 fraction is an integer amount that will be set as the fraction of the money Notes The fraction can only be set if it matches the number of decimal digits for the currency associated with the money . Therefore, for a currency with 2 decimal digits, the maximum for fraction is 99 . Examples iex&gt; Money . put_fraction Money . new ( :USD , &quot;2.49&quot; ) , 99 Money . new ( :USD , &quot;2.99&quot; ) iex&gt; Money . put_fraction Money . new ( :USD , &quot;2.49&quot; ) , 0 Money . new ( :USD , &quot;2.0&quot; ) iex&gt; Money . put_fraction Money . new ( :USD , &quot;2.49&quot; ) , 999 { :error , { Money.InvalidAmountError , &quot;Rounding up to 999 is invalid for currency :USD&quot; } }","ref":"Money.html#put_fraction/2","title":"Money.put_fraction/2","type":"function"},{"doc":"","ref":"Money.html#reduce/1","title":"Money.reduce/1","type":"function"},{"doc":"Round a Money value into the acceptable range for the requested currency. Arguments money is any Money.t/0 opts is a keyword list of options Options :rounding_mode that defines how the number will be rounded. See Decimal.Context . The default is :half_even which is also known as &quot;banker's rounding&quot; :currency_digits which determines the rounding increment. The valid options are :cash , :accounting and :iso or an integer value representing the rounding factor. The default is :iso . Notes There are two kinds of rounding applied: Round to the appropriate number of fractional digits Apply an appropriate rounding increment. Most currencies round to the same precision as the number of decimal digits, but some such as :CHF round to a minimum such as 0.05 when its a cash amount. The rounding increment is applied when the option :currency_digits is set to :cash Digital Tokens (crypto currencies) do not have formal definitions of decimal digits or rounding strategies. Therefore the money is returned unmodified. Examples iex&gt; Money . round Money . new ( &quot;123.73&quot; , :CHF ) , currency_digits : :cash Money . new ( :CHF , &quot;123.75&quot; ) iex&gt; Money . round Money . new ( &quot;123.73&quot; , :CHF ) , currency_digits : 0 Money . new ( :CHF , &quot;124&quot; ) iex&gt; Money . round Money . new ( &quot;123.7456&quot; , :CHF ) Money . new ( :CHF , &quot;123.75&quot; ) iex&gt; Money . round Money . new ( &quot;123.7456&quot; , :JPY ) Money . new ( :JPY , &quot;124&quot; )","ref":"Money.html#round/2","title":"Money.round/2","type":"function"},{"doc":"Split a Money value into a number of parts maintaining the currency's precision and rounding and ensuring that the parts sum to the original amount. Arguments money is any Money.t/0 parts is an integer number of parts into which the money is split Returns a tuple {dividend, remainder} as the function result derived as follows: Round the money amount to the required currency precision using Money.round/1 Divide the result of step 1 by the integer divisor Round the result of the division to the precision of the currency using Money.round/1 Return two numbers: the result of the division and any remainder that could not be applied given the precision of the currency. Examples Money . split Money . new ( 123.5 , :JPY ) , 3 { ¥ 41 , ¥ 1 } Money . split Money . new ( 123.4 , :JPY ) , 3 { ¥ 41 , ¥ 0 } Money . split Money . new ( 123.7 , :USD ) , 9 { $ 13.74 , $ 0.04 }","ref":"Money.html#split/2","title":"Money.split/2","type":"function"},{"doc":"Subtract one Money value struct from another. Options money_1 and money_2 are any valid Money.t/0 types returned by Money.new/2 Returns {:ok, money} or {:error, reason} Example iex&gt; Money . sub Money . new ( :USD , 200 ) , Money . new ( :USD , 100 ) { :ok , Money . new ( :USD , 100 ) }","ref":"Money.html#sub/2","title":"Money.sub/2","type":"function"},{"doc":"Subtract one Money value struct from another and raise on error. Returns either {:ok, money} or {:error, reason} . Arguments money_1 and money_2 are any valid Money.t/0 types returned by Money.new/2 Returns a Money.t/0 struct or raises an exception Examples iex&gt; Money . sub! Money . new ( :USD , 200 ) , Money . new ( :USD , 100 ) Money . new ( :USD , &quot;100&quot; ) Money . sub! Money . new ( :USD , 200 ) , Money . new ( :CAD , 500 ) ** (ArgumentError) Cannot subtract monies with different currencies. Received :USD and :CAD.","ref":"Money.html#sub!/2","title":"Money.sub!/2","type":"function"},{"doc":"Sum a list of monies that may be in different currencies. Arguments money_list is a list of any valid Money.t/0 types returned by Money.new/2 rates is a map of exchange rates. The default is %{} . Money.ExchangeRates.latest_rates/0 can be used to return the latest known exchange rates which can then applied as the rates parameter. Returns {:ok, money} representing the sum of the maybe converted money amounts. The currency of the sum is the currency of the first Money in the money_list . {:error, {exception, reason}} describing an error. Examples iex&gt; Money . sum [ Money . new ( :USD , 100 ) , Money . new ( :USD , 200 ) , Money . new ( :USD , 50 ) ] { :ok , Money . new ( :USD , 350 ) } iex&gt; Money . sum [ Money . new ( :USD , 100 ) , Money . new ( :USD , 200 ) , Money . new ( :AUD , 50 ) ] { :error , { Money.ExchangeRateError , &quot;No exchange rate is available for currency :AUD&quot; } } iex&gt; rates = %{ AUD : Decimal . new ( 2 ) , USD : Decimal . new ( 1 ) } iex&gt; Money . sum [ Money . new ( :USD , 100 ) , Money . new ( :USD , 200 ) , Money . new ( :AUD , 50 ) ] , rates { :ok , Money . from_float ( :USD , 325.0 ) }","ref":"Money.html#sum/2","title":"Money.sum/2","type":"function"},{"doc":"Convert money from one currency to another. Arguments money is any Money.t/0 struct returned by Cldr.Currency.new/2 to_currency is a valid currency code into which the money is converted rates is a Map of currency rates where the map key is an upcased atom or string and the value is a Decimal conversion factor. The default is the latest available exchange rates returned from Money.ExchangeRates.latest_rates() Converting to a currency defined in a locale To convert a Money to a currency defined by a locale, Cldr.Currency.currency_from_locale/1 can be called with a t:Cldr.LanguageTag.t() parameter. It will return the currency configured for that locale. Examples iex&gt; Money . to_currency ( Money . new ( :USD , 100 ) , :AUD , ...&gt; %{ USD : Decimal . new ( 1 ) , AUD : Decimal . from_float ( 0.7345 ) } ) { :ok , Money . new ( :AUD , &quot;73.4500&quot; ) } iex&gt; Money . to_currency ( Money . new ( &quot;USD&quot; , 100 ) , &quot;AUD&quot; , ...&gt; %{ &quot;USD&quot; =&gt; Decimal . new ( 1 ) , &quot;AUD&quot; =&gt; Decimal . from_float ( 0.7345 ) } ) { :ok , Money . new ( :AUD , &quot;73.4500&quot; ) } iex&gt; Money . to_currency ( Money . new ( :USD , 100 ) , :AUDD , ...&gt; %{ USD : Decimal . new ( 1 ) , AUD : Decimal . from_float ( 0.7345 ) } ) { :error , { Cldr.UnknownCurrencyError , &quot;The currency :AUDD is invalid&quot; } } iex&gt; Money . to_currency ( Money . new ( :USD , 100 ) , :CHF , ...&gt; %{ USD : Decimal . new ( 1 ) , AUD : Decimal . from_float ( 0.7345 ) } ) { :error , { Money.ExchangeRateError , &quot;No exchange rate is available for currency :CHF&quot; } }","ref":"Money.html#to_currency/3","title":"Money.to_currency/3","type":"function"},{"doc":"Convert money from one currency to another and raises on error Arguments money is any Money.t/0 struct returned by Cldr.Currency.new/2 to_currency is a valid currency code into which the money is converted rates is a Map of currency rates where the map key is an upcased atom or string and the value is a Decimal conversion factor. The default is the latest available exchange rates returned from Money.ExchangeRates.latest_rates() Examples iex&gt; Money . to_currency! Money . new ( :USD , 100 ) , :AUD , ...&gt; %{ USD : Decimal . new ( 1 ) , AUD : Decimal . from_float ( 0.7345 ) } Money . new ( :AUD , &quot;73.4500&quot; ) iex&gt; Money . to_currency! Money . new ( &quot;USD&quot; , 100 ) , &quot;AUD&quot; , ...&gt; %{ &quot;USD&quot; =&gt; Decimal . new ( 1 ) , &quot;AUD&quot; =&gt; Decimal . from_float ( 0.7345 ) } Money . new ( :AUD , &quot;73.4500&quot; ) =&gt; Money . to_currency! Money . new ( :USD , 100 ) , :ZZZ , %{ USD : Decimal . new ( 1 ) , AUD : Decimal . from_float ( 0.7345 ) } ** (Cldr.UnknownCurrencyError) Currency :ZZZ is not known","ref":"Money.html#to_currency!/3","title":"Money.to_currency!/3","type":"function"},{"doc":"Returns the currecny code of a Money type as an atom . Arguments money is any valid Money.t/0 type returned by Money.new/2 Returns the currency code as an t:atom Example iex&gt; m = Money . new ( &quot;USD&quot; , 100 ) iex&gt; Money . to_currency_code ( m ) :USD","ref":"Money.html#to_currency_code/1","title":"Money.to_currency_code/1","type":"function"},{"doc":"Returns the amount part of a Money type as a Decimal Arguments money is any valid Money.t/0 type returned by Money.new/2 Returns a Decimal.t Example iex&gt; m = Money . new ( &quot;USD&quot; , 100 ) iex&gt; Money . to_decimal ( m ) Decimal . new ( 100 )","ref":"Money.html#to_decimal/1","title":"Money.to_decimal/1","type":"function"},{"doc":"Returns a tuple comprising the currency code, integer amount, exponent and remainder Some services require submission of money items as an integer with an implied exponent that is appropriate to the currency. Rather than return only the integer, Money.to_integer_exp returns the currency code, integer, exponent and remainder. The remainder is included because to return an integer money with an implied exponent the Money has to be rounded potentially leaving a remainder. Options money is any Money.t/0 struct returned by Cldr.Currency.new/2 Notes Since the returned integer is expected to have the implied fractional digits the Money needs to be rounded which is what this function does. Example iex&gt; m = Money . new ( :USD , &quot;200.012356&quot; ) Money . new ( :USD , &quot;200.012356&quot; ) iex&gt; Money . to_integer_exp ( m ) { :USD , 20001 , - 2 , Money . new ( :USD , &quot;0.002356&quot; ) } iex&gt; m = Money . new ( :USD , &quot;200.00&quot; ) Money . new ( :USD , &quot;200.00&quot; ) iex&gt; Money . to_integer_exp ( m ) { :USD , 20000 , - 2 , Money . new ( :USD , &quot;0.00&quot; ) }","ref":"Money.html#to_integer_exp/2","title":"Money.to_integer_exp/2","type":"function"},{"doc":"Returns a formatted string representation of a Money.t/0 . Formatting is performed according to the rules defined by CLDR. See Cldr.Number.to_string/2 for formatting options. The default is to format as a currency which applies the appropriate rounding and fractional digits for the currency. Arguments money is any valid Money.t/0 type returned by Money.new/2 options is a keyword list of options or a %Cldr.Number.Format.Options{} struct Returns {:ok, string} or {:error, reason} Options :backend is any CLDR backend module. The default is Money.default_backend() . currency_symbol : Allows overriding a currency symbol. The alternatives are: :iso the ISO currency code will be used instead of the default currency symbol. :narrow uses the narrow symbol defined for the locale. The same narrow symbol can be defined for more than one currency and therefore this should be used with care. If no narrow symbol is defined, the standard symbol is used. :symbol uses the standard symbol defined in CLDR. A symbol is unique for each currency and can be safely used. &quot;string&quot; uses string as the currency symbol :standard (the default and recommended) uses the CLDR-defined symbol based upon the currency format for the locale. Any other options are passed to Cldr.Number.to_string/3 Examples iex&gt; Money . to_string Money . new ( :USD , 1234 ) { :ok , &quot;$1,234.00&quot; } iex&gt; Money . to_string Money . new ( :JPY , 1234 ) { :ok , &quot;¥1,234&quot; } iex&gt; Money . to_string Money . new ( :THB , 1234 ) { :ok , &quot;THB 1,234.00&quot; } iex&gt; Money . to_string Money . new ( :THB , 1234 , fractional_digits : 4 ) { :ok , &quot;THB 1,234.0000&quot; } iex&gt; Money . to_string Money . new ( :USD , 1234 ) , format : :long { :ok , &quot;1,234 US dollars&quot; }","ref":"Money.html#to_string/2","title":"Money.to_string/2","type":"function"},{"doc":"Returns a formatted string representation of a Money.t/0 or raises if there is an error. Formatting is performed according to the rules defined by CLDR. See Cldr.Number.to_string!/2 for formatting options. The default is to format as a currency which applies the appropriate rounding and fractional digits for the currency. Arguments money is any valid Money.t/0 type returned by Money.new/2 options is a keyword list of options or a %Cldr.Number.Format.Options{} struct Options :backend is any CLDR backend module. The default is Money.default_backend() . Any other options are passed to Cldr.Number.to_string/3 Examples iex&gt; Money . to_string! Money . new ( :USD , 1234 ) &quot;$1,234.00&quot; iex&gt; Money . to_string! Money . new ( :JPY , 1234 ) &quot;¥1,234&quot; iex&gt; Money . to_string! Money . new ( :THB , 1234 ) &quot;THB 1,234.00&quot; iex&gt; Money . to_string! Money . new ( :USD , 1234 ) , format : :long &quot;1,234 US dollars&quot;","ref":"Money.html#to_string!/2","title":"Money.to_string!/2","type":"function"},{"doc":"Return a zero amount Money.t/0 in the given currency. Arguments money_or_currency is either a Money.t/0 or a currency code options is a keyword list of options passed to Money.new/3 . The default is [] . Example iex&gt; Money . zero ( :USD ) Money . new ( :USD , &quot;0&quot; ) iex&gt; money = Money . new ( :USD , 200 ) iex&gt; Money . zero ( money ) Money . new ( :USD , &quot;0&quot; ) iex&gt; Money . zero :ZZZ { :error , { Cldr.UnknownCurrencyError , &quot;The currency :ZZZ is invalid&quot; } }","ref":"Money.html#zero/2","title":"Money.zero/2","type":"function"},{"doc":"Returns a boolean indicating if Money.t/0 has a zero value. Arguments money is any valid Money.t/0 type returned by Money.new/2 Example iex&gt; Money . zero? ( Money . new ( :USD , 0 ) ) true iex&gt; Money . zero? ( Money . new ( :USD , 1 ) ) false iex&gt; Money . zero? ( Money . new ( :USD , - 1 ) ) false","ref":"Money.html#zero?/1","title":"Money.zero?/1","type":"function"},{"doc":"An amount can be expressed as a float, an integer, a Decimal or a string (which is converted to a Decimal)","ref":"Money.html#t:amount/0","title":"Money.amount/0","type":"type"},{"doc":"A currency code is an ISO 4217 code expressed as an atom or binary or an ISO 24165 Digital Token ID or Digital Token short name.","ref":"Money.html#t:currency_code/0","title":"Money.currency_code/0","type":"type"},{"doc":"Money is composed of an atom representation of an ISO4217 currency code and a Decimal representation of an amount.","ref":"Money.html#t:t/0","title":"Money.t/0","type":"type"},{"doc":"","ref":"Money.Application.html","title":"Money.Application","type":"module"},{"doc":"Callback implementation for Application.start/2 .","ref":"Money.Application.html#start/2","title":"Money.Application.start/2","type":"function"},{"doc":"Functions to return lists of known, historic and legal tender currencies.","ref":"Money.Currency.html","title":"Money.Currency","type":"module"},{"doc":"","ref":"Money.Currency.html#currency_for_code/1","title":"Money.Currency.currency_for_code/1","type":"function"},{"doc":"Returns the list of currently active ISO 4217 currency codes. Example: iex&gt; Money.Currency . known_current_currencies [ :AED , :AFN , :ALL , :AMD , :ANG , :AOA , :ARS , :AUD , :AWG , :AZN , :BAM , :BBD , :BDT , :BGN , :BHD , :BIF , :BMD , :BND , :BOB , :BOV , :BRL , :BSD , :BTN , :BWP , :BYN , :BZD , :CAD , :CDF , :CHE , :CHF , :CHW , :CLF , :CLP , :CNY , :COP , :COU , :CRC , :CUC , :CUP , :CVE , :CZK , :DJF , :DKK , :DOP , :DZD , :EGP , :ERN , :ETB , :EUR , :FJD , :FKP , :GBP , :GEL , :GHS , :GIP , :GMD , :GNF , :GTQ , :GYD , :HKD , :HNL , :HRK , :HTG , :HUF , :IDR , :ILS , :INR , :IQD , :IRR , :ISK , :JMD , :JOD , :JPY , :KES , :KGS , :KHR , :KMF , :KPW , :KRW , :KWD , :KYD , :KZT , :LAK , :LBP , :LKR , :LRD , :LSL , :LYD , :MAD , :MDL , :MGA , :MKD , :MMK , :MNT , :MOP , :MRU , :MUR , :MVR , :MWK , :MXN , :MXV , :MYR , :MZN , :NAD , :NGN , :NIO , :NOK , :NPR , :NZD , :OMR , :PAB , :PEN , :PGK , :PHP , :PKR , :PLN , :PYG , :QAR , :RON , :RSD , :RUB , :RWF , :SAR , :SBD , :SCR , :SDG , :SEK , :SGD , :SHP , :SLL , :SOS , :SRD , :SSP , :STN , :SVC , :SYP , :SZL , :THB , :TJS , :TMT , :TND , :TOP , :TRY , :TTD , :TWD , :TZS , :UAH , :UGX , :USD , :USN , :UYI , :UYU , :UYW , :UZS , :VES , :VND , :VUV , :WST , :XAF , :XAG , :XAU , :XBA , :XBB , :XBC , :XBD , :XCD , :XDR , :XOF , :XPD , :XPF , :XPT , :XSU , :XTS , :XUA , :XXX , :YER , :ZAR , :ZMW , :ZWL ]","ref":"Money.Currency.html#known_current_currencies/0","title":"Money.Currency.known_current_currencies/0","type":"function"},{"doc":"Returns the list of historic ISO 4217 currency codes. Example: iex&gt; Money.Currency . known_historic_currencies [ :ADP , :AFA , :ALK , :AOK , :AON , :AOR , :ARA , :ARL , :ARM , :ARP , :ATS , :AZM , :BAD , :BAN , :BEC , :BEF , :BEL , :BGL , :BGM , :BGO , :BOL , :BOP , :BRB , :BRC , :BRE , :BRN , :BRR , :BRZ , :BUK , :BYB , :BYR , :CLE , :CNH , :CNX , :CSD , :CSK , :CYP , :DDM , :DEM , :ECS , :ECV , :EEK , :ESA , :ESB , :ESP , :FIM , :FRF , :GEK , :GHC , :GNS , :GQE , :GRD , :GWE , :GWP , :HRD , :IEP , :ILP , :ILR , :ISJ , :ITL , :KRH , :KRO , :LTL , :LTT , :LUC , :LUF , :LUL , :LVL , :LVR , :MAF , :MCF , :MDC , :MGF , :MKN , :MLF , :MRO , :MTL , :MTP , :MVP , :MXP , :MZE , :MZM , :NIC , :NLG , :PEI , :PES , :PLZ , :PTE , :RHD , :ROL , :RUR , :SDD , :SDP , :SIT , :SKK , :SLE , :SRG , :STD , :SUR , :TJR , :TMM , :TPE , :TRL , :UAK , :UGS , :USS , :UYP , :VEB , :VED , :VEF , :VNN , :XEU , :XFO , :XFU , :XRE , :YDD , :YUD , :YUM , :YUN , :YUR , :ZAL , :ZMK , :ZRN , :ZRZ , :ZWD , :ZWR ]","ref":"Money.Currency.html#known_historic_currencies/0","title":"Money.Currency.known_historic_currencies/0","type":"function"},{"doc":"Returns the list of legal tender ISO 4217 currency codes. Example: iex&gt; Money.Currency . known_tender_currencies [ :ADP , :AED , :AFA , :AFN , :ALK , :ALL , :AMD , :ANG , :AOA , :AOK , :AON , :AOR , :ARA , :ARL , :ARM , :ARP , :ARS , :ATS , :AUD , :AWG , :AZM , :AZN , :BAD , :BAM , :BAN , :BBD , :BDT , :BEC , :BEF , :BEL , :BGL , :BGM , :BGN , :BGO , :BHD , :BIF , :BMD , :BND , :BOB , :BOL , :BOP , :BOV , :BRB , :BRC , :BRE , :BRL , :BRN , :BRR , :BRZ , :BSD , :BTN , :BUK , :BWP , :BYB , :BYN , :BYR , :BZD , :CAD , :CDF , :CHE , :CHF , :CHW , :CLE , :CLF , :CLP , :CNH , :CNX , :CNY , :COP , :COU , :CRC , :CSD , :CSK , :CUC , :CUP , :CVE , :CYP , :CZK , :DDM , :DEM , :DJF , :DKK , :DOP , :DZD , :ECS , :ECV , :EEK , :EGP , :ERN , :ESA , :ESB , :ESP , :ETB , :EUR , :FIM , :FJD , :FKP , :FRF , :GBP , :GEK , :GEL , :GHC , :GHS , :GIP , :GMD , :GNF , :GNS , :GQE , :GRD , :GTQ , :GWE , :GWP , :GYD , :HKD , :HNL , :HRD , :HRK , :HTG , :HUF , :IDR , :IEP , :ILP , :ILR , :ILS , :INR , :IQD , :IRR , :ISJ , :ISK , :ITL , :JMD , :JOD , :JPY , :KES , :KGS , :KHR , :KMF , :KPW , :KRH , :KRO , :KRW , :KWD , :KYD , :KZT , :LAK , :LBP , :LKR , :LRD , :LSL , :LTL , :LTT , :LUC , :LUF , :LUL , :LVL , :LVR , :LYD , :MAD , :MAF , :MCF , :MDC , :MDL , :MGA , :MGF , :MKD , :MKN , :MLF , :MMK , :MNT , :MOP , :MRO , :MRU , :MTL , :MTP , :MUR , :MVP , :MVR , :MWK , :MXN , :MXP , :MXV , :MYR , :MZE , :MZM , :MZN , :NAD , :NGN , :NIC , :NIO , :NLG , :NOK , :NPR , :NZD , :OMR , :PAB , :PEI , :PEN , :PES , :PGK , :PHP , :PKR , :PLN , :PLZ , :PTE , :PYG , :QAR , :RHD , :ROL , :RON , :RSD , :RUB , :RUR , :RWF , :SAR , :SBD , :SCR , :SDD , :SDG , :SDP , :SEK , :SGD , :SHP , :SIT , :SKK , :SLE , :SLL , :SOS , :SRD , :SRG , :SSP , :STD , :STN , :SUR , :SVC , :SYP , :SZL , :THB , :TJR , :TJS , :TMM , :TMT , :TND , :TOP , :TPE , :TRL , :TRY , :TTD , :TWD , :TZS , :UAH , :UAK , :UGS , :UGX , :USD , :USN , :USS , :UYI , :UYP , :UYU , :UYW , :UZS , :VEB , :VED , :VEF , :VES , :VND , :VNN , :VUV , :WST , :XAF , :XAG , :XAU , :XBA , :XBB , :XBC , :XBD , :XCD , :XDR , :XEU , :XFO , :XFU , :XOF , :XPD , :XPF , :XPT , :XRE , :XSU , :XTS , :XUA , :XXX , :YDD , :YER , :YUD , :YUM , :YUN , :YUR , :ZAL , :ZAR , :ZMK , :ZMW , :ZRN , :ZRZ , :ZWD , :ZWL , :ZWR ]","ref":"Money.Currency.html#known_tender_currencies/0","title":"Money.Currency.known_tender_currencies/0","type":"function"},{"doc":"A set of financial functions, primarily related to discounted cash flows. Some of the algorithms are from finance formulas","ref":"Money.Financial.html","title":"Money.Financial","type":"module"},{"doc":"Calculates the future value for a list of cash flows and an interest rate. flows is a list of tuples representing a cash flow. Each flow is represented as a tuple of the form {period, %Money{}} interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 Example iex&gt; Money.Financial . future_value ( [ { 4 , Money . new ( :USD , 10000 ) } , { 5 , Money . new ( :USD , 10000 ) } , { 6 , Money . new ( :USD , 10000 ) } ] , 0.13 ) Money . new ( :USD , &quot;34068.99999999999999999999999&quot; ) iex&gt; Money.Financial . future_value [ { 0 , Money . new ( :USD , 5000 ) } , { 1 , Money . new ( :USD , 2000 ) } ] , 0.12 Money . new ( :USD , &quot;7600.000000000000000000000000&quot; )","ref":"Money.Financial.html#future_value/2","title":"Money.Financial.future_value/2","type":"function"},{"doc":"Calculates the future value for a present value, an interest rate and a number of periods. present_value is a %Money{} representation of the present value interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 periods in an integer number of periods Examples iex&gt; Money.Financial . future_value Money . new ( :USD , 10000 ) , 0.08 , 1 Money . new ( :USD , &quot;10800.00&quot; ) iex&gt; Money.Financial . future_value Money . new ( :USD , 10000 ) , 0.04 , 2 Money . new ( :USD , &quot;10816.0000&quot; ) iex&gt; Money.Financial . future_value Money . new ( :USD , 10000 ) , 0.02 , 4 Money . new ( :USD , &quot;10824.32160000&quot; )","ref":"Money.Financial.html#future_value/3","title":"Money.Financial.future_value/3","type":"function"},{"doc":"Calculates the effective interest rate for a given present value, a future value and a number of periods. present_value is a %Money{} representation of the present value future_value is a %Money{} representation of the future value periods is an integer number of a period Examples iex&gt; Money.Financial . interest_rate Money . new ( :USD , 10000 ) , Money . new ( :USD , 10816 ) , 2 Decimal . new ( &quot;0.04&quot; ) iex&gt; Money.Financial . interest_rate Money . new ( :USD , 10000 ) , Money . new ( :USD , &quot;10824.3216&quot; ) , 4 Decimal . new ( &quot;0.02&quot; )","ref":"Money.Financial.html#interest_rate/3","title":"Money.Financial.interest_rate/3","type":"function"},{"doc":"Calculates the interal rate of return for a given list of cash flows. flows is a list of tuples representing a cash flow. Each flow is represented as a tuple of the form {period, %Money{}}","ref":"Money.Financial.html#internal_rate_of_return/1","title":"Money.Financial.internal_rate_of_return/1","type":"function"},{"doc":"Calculates the net present value of an initial investment, a list of cash flows and an interest rate. flows is a list of tuples representing a cash flow. Each flow is represented as a tuple of the form {period, %Money{}} interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 investment is a %Money{} struct representing the initial investment Example iex&gt; flows = [ { 0 , Money . new ( :USD , 5000 ) } , { 1 , Money . new ( :USD , 2000 ) } , { 2 , Money . new ( :USD , 500 ) } , { 3 , Money . new ( :USD , 10_000 ) } ] iex&gt; Money.Financial . net_present_value flows , 0.08 , Money . new ( :USD , 100 ) Money . new ( :USD , &quot;15118.84367220444038002337042&quot; ) iex&gt; Money.Financial . net_present_value flows , 0.08 Money . new ( :USD , &quot;15218.84367220444038002337042&quot; )","ref":"Money.Financial.html#net_present_value/2","title":"Money.Financial.net_present_value/2","type":"function"},{"doc":"Calculates the net present value of an initial investment, a recurring payment, an interest rate and a number of periods investment is a %Money{} struct representing the initial investment future_value is a %Money{} representation of the future value interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 periods in an integer number of a period Example iex&gt; Money.Financial . net_present_value Money . new ( :USD , 10000 ) , 0.13 , 2 Money . new ( :USD , &quot;7831.466833737959119743127888&quot; ) iex&gt; Money.Financial . net_present_value Money . new ( :USD , 10000 ) , 0.13 , 2 , Money . new ( :USD , 100 ) Money . new ( :USD , &quot;7731.466833737959119743127888&quot; )","ref":"Money.Financial.html#net_present_value/3","title":"Money.Financial.net_present_value/3","type":"function"},{"doc":"","ref":"Money.Financial.html#net_present_value/4","title":"Money.Financial.net_present_value/4","type":"function"},{"doc":"Calculates the payment for a given loan or annuity given a present value, an interest rate and a number of periods. present_value is a %Money{} representation of the present value interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 periods is an integer number of periods Example iex&gt; Money.Financial . payment Money . new ( :USD , 100 ) , 0.12 , 20 Money . new ( :USD , &quot;13.38787800396606622792492299&quot; )","ref":"Money.Financial.html#payment/3","title":"Money.Financial.payment/3","type":"function"},{"doc":"Calculates the number of periods between a present value and a future value with a given interest rate. present_value is a %Money{} representation of the present value future_value is a %Money{} representation of the future value interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 Example iex&gt; Money.Financial . periods Money . new ( :USD , 1500 ) , Money . new ( :USD , 2000 ) , 0.005 Decimal . new ( &quot;57.68013595323872502502238648&quot; )","ref":"Money.Financial.html#periods/3","title":"Money.Financial.periods/3","type":"function"},{"doc":"Calculates the present value for a list of cash flows and an interest rate. flows is a list of tuples representing a cash flow. Each flow is represented as a tuple of the form {period, %Money{}} interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 Example iex&gt; Money.Financial . present_value ( [ { 4 , Money . new ( :USD , 10000 ) } , { 5 , Money . new ( :USD , 10000 ) } , { 6 , Money . new ( :USD , 10000 ) } ] , 0.13 ) Money . new ( :USD , &quot;16363.97191111964880256655144&quot; ) iex&gt; Money.Financial . present_value [ { 0 , Money . new ( :USD , - 1000 ) } , { 1 , Money . new ( :USD , - 4000 ) } ] , 0.1 Money . new ( :USD , &quot;-4636.363636363636363636363636&quot; )","ref":"Money.Financial.html#present_value/2","title":"Money.Financial.present_value/2","type":"function"},{"doc":"Calculates the present value for future value, an interest rate and a number of periods future_value is a %Money{} representation of the future value interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 periods in an integer number of periods Examples iex&gt; Money.Financial . present_value Money . new ( :USD , 100 ) , 0.08 , 2 Money . new ( :USD , &quot;85.73388203017832647462277092&quot; ) iex&gt; Money.Financial . present_value Money . new ( :USD , 1000 ) , 0.10 , 20 Money . new ( :USD , &quot;148.6436280241436864020760472&quot; )","ref":"Money.Financial.html#present_value/3","title":"Money.Financial.present_value/3","type":"function"},{"doc":"","ref":"Money.Sigil.html","title":"Money.Sigil","type":"module"},{"doc":"Implements the sigil ~M for Money The lower case ~m variant does not exist as interpolation and excape characters are not useful for Money sigils. Example iex&gt; import Money.Sigil iex&gt; ~M[1000]usd Money . new ( :USD , &quot;1000&quot; ) iex&gt; ~M[1000.34]usd Money . new ( :USD , &quot;1000.34&quot; )","ref":"Money.Sigil.html#sigil_M/2","title":"Money.Sigil.sigil_M/2","type":"function"},{"doc":"","ref":"Money.Sigil.html#validate_currency!/1","title":"Money.Sigil.validate_currency!/1","type":"function"},{"doc":"Implements a behaviour and functions to retrieve exchange rates from an exchange rate service. Configuration for the exchange rate service is defined in a Money.ExchangeRates.Config struct. A default configuration is returned by Money.ExchangeRates.default_config/0 . The default configuration is: config :ex_money , exchange_rate_service : false , exchange_rates_retrieve_every : 300_000 , api_module : Money.ExchangeRates.OpenExchangeRates , callback_module : Money.ExchangeRates.Callback , preload_historic_rates : nil log_failure : :warn , log_info : :info , log_success : nil These keys are are defined as follows: :exchange_rate_service is a boolean that determines whether to automatically start the exchange rate retrieval service. The default it false. :exchange_rates_retrieve_every defines how often the exchange rates are retrieved in milliseconds. The default is 5 minutes (300,000 milliseconds) :api_module identifies the module that does the retrieval of exchange rates. This is any module that implements the Money.ExchangeRates behaviour. The default is Money.ExchangeRates.OpenExchangeRates :callback_module defines a module that follows the Money.ExchangeRates.Callback behaviour whereby the function rates_retrieved/2 is invoked after every successful retrieval of exchange rates. The default is Money.ExchangeRates.Callback . :preload_historic_rates defines a date or a date range, that will be requested when the exchange rate service starts up. The date or date range should be specified as either a Date.t or a Date.Range.t or a tuple of {Date.t, Date.t} representing the from and to dates for the rates to be retrieved. The default is nil meaning no historic rates are preloaded. :log_failure defines the log level at which api retrieval errors are logged. The default is :warn :log_success defines the log level at which successful api retrieval notifications are logged. The default is nil which means no logging. :log_info defines the log level at which service startup messages are logged. The default is :info . :retriever_options is available for exchange rate retriever module developers as a place to add retriever-specific configuration information. This information should be added in the init/1 callback in the retriever module. See Money.ExchangeRates.OpenExchangeRates.init/1 for an example. Keys can also be configured to retrieve values from environment variables. This lookup is done at runtime to facilitate deployment strategies. If the value of a configuration key is {:system, &quot;some_string&quot;} then &quot;some_string&quot; is interpreted as an environment variable name which is passed to System.get_env/2. An example configuration might be: config :ex_money , exchange_rate_service : { :system , &quot;RATE_SERVICE&quot; } , exchange_rates_retrieve_every : { :system , &quot;RETRIEVE_EVERY&quot; } , Open Exchange Rates If you plan to use the provided Open Exchange Rates module to retrieve exchange rates then you should also provide the additional configuration key for app_id : config :ex_money , open_exchange_rates_app_id : &quot;your_app_id&quot; or configure it via environment variable: config :ex_money , open_exchange_rates_app_id : { :system , &quot;OPEN_EXCHANGE_RATES_APP_ID&quot; } The default exchange rate retrieval module is provided in Money.ExchangeRates.OpenExchangeRates which can be used as a example to implement your own retrieval module for other services. Managing the configuration at runtime During exchange rate service startup, the function init/1 is called on the configuration exchange rate retrieval module. This module is expected to return an updated configuration allowing a developer to customise how the configuration is to be managed. See the implementation at Money.ExchangeRates.OpenExchangeRates.init/1 for an example.","ref":"Money.ExchangeRates.html","title":"Money.ExchangeRates","type":"behaviour"},{"doc":"Returns the configuration for ex_money including the configuration merged from the configured exchange rates retriever module.","ref":"Money.ExchangeRates.html#config/0","title":"Money.ExchangeRates.config/0","type":"function"},{"doc":"Decode the body returned from the API request and return a map of rates. THe map of rates must have an upcased atom key representing an ISO 4217 currency code and the value must be a Decimal number.","ref":"Money.ExchangeRates.html#c:decode_rates/1","title":"Money.ExchangeRates.decode_rates/1","type":"callback"},{"doc":"Returns the default configuration for the exchange rates retriever.","ref":"Money.ExchangeRates.html#default_config/0","title":"Money.ExchangeRates.default_config/0","type":"function"},{"doc":"Invoked to return the historic exchange rates from the configured exchange rate retrieval service. config is an %Money.ExchangeRataes.Config{} struct Returns {:ok, map_of_historic_rates} or {:error, reason}","ref":"Money.ExchangeRates.html#c:get_historic_rates/2","title":"Money.ExchangeRates.get_historic_rates/2","type":"callback"},{"doc":"Invoked to return the latest exchange rates from the configured exchange rate retrieval service. config is an %Money.ExchangeRataes.Config{} struct Returns {:ok, map_of_rates} or {:error, reason}","ref":"Money.ExchangeRates.html#c:get_latest_rates/1","title":"Money.ExchangeRates.get_latest_rates/1","type":"callback"},{"doc":"Return historic exchange rates. date is a date returned by Date.new/3 or any struct with the elements :year , :month and :day . Returns: {:ok, rates} if exchange rates are successfully retrieved. rates is a map of exchange rates. {:error, reason} if no exchange rates can be returned. Note; all dates are expected to be in the Calendar.ISO calendar This function looks up the historic exchange rates in a an ETS table called :exchange_rates . The actual retrieval of rates is requested through Money.ExchangeRates.Retriever.historic_rates/1 .","ref":"Money.ExchangeRates.html#historic_rates/1","title":"Money.ExchangeRates.historic_rates/1","type":"function"},{"doc":"Given the default configuration, returns an updated configuration at runtime during exchange rates service startup. This callback is optional. If the callback is not defined, the default configuration returned by Money.ExchangeRates.default_config/0 is used. config is the configuration returned by Money.ExchangeRates.default_config/0 The callback is expected to return a %Money.ExchangeRates.Config.t() struct which may have been updated. The configuration key :retriever_options is available for any service-specific configuration.","ref":"Money.ExchangeRates.html#c:init/1","title":"Money.ExchangeRates.init/1","type":"callback"},{"doc":"Return the timestamp of the last successful retrieval of exchange rates or {:error, reason} if no timestamp is known. Example Money.ExchangeRates . last_updated #&gt; {:ok, % DateTime { calendar : Calendar.ISO , day : 20 , hour : 12 , microsecond : { 731942 , 6 } , minute : 36 , month : 11 , second : 6 , std_offset : 0 , time_zone : &quot;Etc/UTC&quot; , utc_offset : 0 , year : 2016 , zone_abbr : &quot;UTC&quot; } }","ref":"Money.ExchangeRates.html#last_updated/0","title":"Money.ExchangeRates.last_updated/0","type":"function"},{"doc":"Return the latest exchange rates. Returns: {:ok, rates} if exchange rates are successfully retrieved. rates is a map of exchange rates. {:error, reason} if no exchange rates can be returned. This function looks up the latest exchange rates in a an ETS table called :exchange_rates . The actual retrieval of rates is requested through Money.ExchangeRates.Retriever.latest_rates/0 .","ref":"Money.ExchangeRates.html#latest_rates/0","title":"Money.ExchangeRates.latest_rates/0","type":"function"},{"doc":"Returns true if the latest exchange rates are available and false otherwise.","ref":"Money.ExchangeRates.html#latest_rates_available?/0","title":"Money.ExchangeRates.latest_rates_available?/0","type":"function"},{"doc":"","ref":"Money.ExchangeRates.html#t:t/0","title":"Money.ExchangeRates.t/0","type":"type"},{"doc":"Defines a cache behaviour and default inplementation of a cache for exchange rates","ref":"Money.ExchangeRates.Cache.html","title":"Money.ExchangeRates.Cache","type":"behaviour"},{"doc":"","ref":"Money.ExchangeRates.Cache.html#cache/0","title":"Money.ExchangeRates.Cache.cache/0","type":"function"},{"doc":"","ref":"Money.ExchangeRates.Cache.html#historic_rates/1","title":"Money.ExchangeRates.Cache.historic_rates/1","type":"function"},{"doc":"Returns the exchange rates for a given date.","ref":"Money.ExchangeRates.Cache.html#c:historic_rates/1","title":"Money.ExchangeRates.Cache.historic_rates/1","type":"callback"},{"doc":"Initialize the cache when the exchange rates retriever is started","ref":"Money.ExchangeRates.Cache.html#c:init/0","title":"Money.ExchangeRates.Cache.init/0","type":"callback"},{"doc":"","ref":"Money.ExchangeRates.Cache.html#latest_rates/0","title":"Money.ExchangeRates.Cache.latest_rates/0","type":"function"},{"doc":"Retrieve the latest exchange rates from the cache.","ref":"Money.ExchangeRates.Cache.html#c:latest_rates/0","title":"Money.ExchangeRates.Cache.latest_rates/0","type":"callback"},{"doc":"Store the historic exchange rates for a given date in the cache.","ref":"Money.ExchangeRates.Cache.html#c:store_historic_rates/2","title":"Money.ExchangeRates.Cache.store_historic_rates/2","type":"callback"},{"doc":"Store the latest exchange rates in the cache.","ref":"Money.ExchangeRates.Cache.html#c:store_latest_rates/2","title":"Money.ExchangeRates.Cache.store_latest_rates/2","type":"callback"},{"doc":"Terminate the cache when the retriver process stops normally","ref":"Money.ExchangeRates.Cache.html#c:terminate/0","title":"Money.ExchangeRates.Cache.terminate/0","type":"callback"},{"doc":"Money.ExchangeRates.Cache implementation for :dets","ref":"Money.ExchangeRates.Cache.Dets.html","title":"Money.ExchangeRates.Cache.Dets","type":"module"},{"doc":"","ref":"Money.ExchangeRates.Cache.Dets.html#get/1","title":"Money.ExchangeRates.Cache.Dets.get/1","type":"function"},{"doc":"Callback implementation for Money.ExchangeRates.Cache.historic_rates/1 .","ref":"Money.ExchangeRates.Cache.Dets.html#historic_rates/1","title":"Money.ExchangeRates.Cache.Dets.historic_rates/1","type":"function"},{"doc":"Callback implementation for Money.ExchangeRates.Cache.init/0 .","ref":"Money.ExchangeRates.Cache.Dets.html#init/0","title":"Money.ExchangeRates.Cache.Dets.init/0","type":"function"},{"doc":"","ref":"Money.ExchangeRates.Cache.Dets.html#last_updated/0","title":"Money.ExchangeRates.Cache.Dets.last_updated/0","type":"function"},{"doc":"Callback implementation for Money.ExchangeRates.Cache.latest_rates/0 .","ref":"Money.ExchangeRates.Cache.Dets.html#latest_rates/0","title":"Money.ExchangeRates.Cache.Dets.latest_rates/0","type":"function"},{"doc":"","ref":"Money.ExchangeRates.Cache.Dets.html#put/2","title":"Money.ExchangeRates.Cache.Dets.put/2","type":"function"},{"doc":"Callback implementation for Money.ExchangeRates.Cache.store_historic_rates/2 .","ref":"Money.ExchangeRates.Cache.Dets.html#store_historic_rates/2","title":"Money.ExchangeRates.Cache.Dets.store_historic_rates/2","type":"function"},{"doc":"Callback implementation for Money.ExchangeRates.Cache.store_latest_rates/2 .","ref":"Money.ExchangeRates.Cache.Dets.html#store_latest_rates/2","title":"Money.ExchangeRates.Cache.Dets.store_latest_rates/2","type":"function"},{"doc":"Callback implementation for Money.ExchangeRates.Cache.terminate/0 .","ref":"Money.ExchangeRates.Cache.Dets.html#terminate/0","title":"Money.ExchangeRates.Cache.Dets.terminate/0","type":"function"},{"doc":"Money.ExchangeRates.Cache implementation for :ets and :dets","ref":"Money.ExchangeRates.Cache.Ets.html","title":"Money.ExchangeRates.Cache.Ets","type":"module"},{"doc":"","ref":"Money.ExchangeRates.Cache.Ets.html#get/1","title":"Money.ExchangeRates.Cache.Ets.get/1","type":"function"},{"doc":"Callback implementation for Money.ExchangeRates.Cache.historic_rates/1 .","ref":"Money.ExchangeRates.Cache.Ets.html#historic_rates/1","title":"Money.ExchangeRates.Cache.Ets.historic_rates/1","type":"function"},{"doc":"Callback implementation for Money.ExchangeRates.Cache.init/0 .","ref":"Money.ExchangeRates.Cache.Ets.html#init/0","title":"Money.ExchangeRates.Cache.Ets.init/0","type":"function"},{"doc":"","ref":"Money.ExchangeRates.Cache.Ets.html#last_updated/0","title":"Money.ExchangeRates.Cache.Ets.last_updated/0","type":"function"},{"doc":"Callback implementation for Money.ExchangeRates.Cache.latest_rates/0 .","ref":"Money.ExchangeRates.Cache.Ets.html#latest_rates/0","title":"Money.ExchangeRates.Cache.Ets.latest_rates/0","type":"function"},{"doc":"","ref":"Money.ExchangeRates.Cache.Ets.html#put/2","title":"Money.ExchangeRates.Cache.Ets.put/2","type":"function"},{"doc":"Callback implementation for Money.ExchangeRates.Cache.store_historic_rates/2 .","ref":"Money.ExchangeRates.Cache.Ets.html#store_historic_rates/2","title":"Money.ExchangeRates.Cache.Ets.store_historic_rates/2","type":"function"},{"doc":"Callback implementation for Money.ExchangeRates.Cache.store_latest_rates/2 .","ref":"Money.ExchangeRates.Cache.Ets.html#store_latest_rates/2","title":"Money.ExchangeRates.Cache.Ets.store_latest_rates/2","type":"function"},{"doc":"Callback implementation for Money.ExchangeRates.Cache.terminate/0 .","ref":"Money.ExchangeRates.Cache.Ets.html#terminate/0","title":"Money.ExchangeRates.Cache.Ets.terminate/0","type":"function"},{"doc":"","ref":"Money.ExchangeRates.Cache.EtsDets.html","title":"Money.ExchangeRates.Cache.EtsDets","type":"module"},{"doc":"","ref":"Money.ExchangeRates.Cache.EtsDets.html#define_common_functions/0","title":"Money.ExchangeRates.Cache.EtsDets.define_common_functions/0","type":"macro"},{"doc":"Default exchange rates retrieval callback module. When exchange rates are successfully retrieved, the function latest_rates_retrieved/2 or historic_rates_retrieved/2 is called to perform any desired serialization or proocessing.","ref":"Money.ExchangeRates.Callback.html","title":"Money.ExchangeRates.Callback","type":"behaviour"},{"doc":"Callback function invoked when historic exchange rates are retrieved.","ref":"Money.ExchangeRates.Callback.html#historic_rates_retrieved/2","title":"Money.ExchangeRates.Callback.historic_rates_retrieved/2","type":"function"},{"doc":"Defines the behaviour to retrieve historic exchange rates from an external data source.","ref":"Money.ExchangeRates.Callback.html#c:historic_rates_retrieved/2","title":"Money.ExchangeRates.Callback.historic_rates_retrieved/2","type":"callback"},{"doc":"Callback function invoked when the latest exchange rates are retrieved.","ref":"Money.ExchangeRates.Callback.html#latest_rates_retrieved/2","title":"Money.ExchangeRates.Callback.latest_rates_retrieved/2","type":"function"},{"doc":"Defines the behaviour to retrieve the latest exchange rates from an external data source.","ref":"Money.ExchangeRates.Callback.html#c:latest_rates_retrieved/2","title":"Money.ExchangeRates.Callback.latest_rates_retrieved/2","type":"callback"},{"doc":"","ref":"Money.ExchangeRates.Config.html","title":"Money.ExchangeRates.Config","type":"module"},{"doc":"","ref":"Money.ExchangeRates.Config.html#t:t/0","title":"Money.ExchangeRates.Config.t/0","type":"type"},{"doc":"Implements the Money.ExchangeRates for the Open Exchange Rates service. Required configuration: The configuration key :open_exchange_rates_app_id should be set to your app_id . for example: config :ex_money , open_exchange_rates_app_id : &quot;your_app_id&quot; or configure it via environment variable: config :ex_money , open_exchange_rates_app_id : { :system , &quot;OPEN_EXCHANGE_RATES_APP_ID&quot; } It is also possible to configure an alternative base url for this service in case it changes in the future. For example: config :ex_money , open_exchange_rates_app_id : &quot;your_app_id&quot; open_exchange_rates_url : &quot;https://openexchangerates.org/alternative_api&quot;","ref":"Money.ExchangeRates.OpenExchangeRates.html","title":"Money.ExchangeRates.OpenExchangeRates","type":"module"},{"doc":"Callback implementation for Money.ExchangeRates.decode_rates/1 .","ref":"Money.ExchangeRates.OpenExchangeRates.html#decode_rates/1","title":"Money.ExchangeRates.OpenExchangeRates.decode_rates/1","type":"function"},{"doc":"Retrieves the historic exchange rates from Open Exchange Rates site. date is a date returned by Date.new/3 or any struct with the elements :year , :month and :day . config is the retrieval configuration. When invoked from the exchange rates services this will be the config returned from Money.ExchangeRates.config/0 Returns: {:ok, rates} if the rates can be retrieved {:error, reason} if rates cannot be retrieved Typically this function is called by the exchange rates retrieval service although it can be called outside that context as required.","ref":"Money.ExchangeRates.OpenExchangeRates.html#get_historic_rates/2","title":"Money.ExchangeRates.OpenExchangeRates.get_historic_rates/2","type":"function"},{"doc":"Retrieves the latest exchange rates from Open Exchange Rates site. config is the retrieval configuration. When invoked from the exchange rates services this will be the config returned from Money.ExchangeRates.config/0 Returns: {:ok, rates} if the rates can be retrieved {:error, reason} if rates cannot be retrieved Typically this function is called by the exchange rates retrieval service although it can be called outside that context as required.","ref":"Money.ExchangeRates.OpenExchangeRates.html#get_latest_rates/1","title":"Money.ExchangeRates.OpenExchangeRates.get_latest_rates/1","type":"function"},{"doc":"Update the retriever configuration to include the requirements for Open Exchange Rates. This function is invoked when the exchange rate service starts up, just after the ets table :exchange_rates is created. default_config is the configuration returned by Money.ExchangeRates.default_config/0 Returns the configuration either unchanged or updated with additional configuration specific to this exchange rates retrieval module.","ref":"Money.ExchangeRates.OpenExchangeRates.html#init/1","title":"Money.ExchangeRates.OpenExchangeRates.init/1","type":"function"},{"doc":"Implements a GenServer to retrieve exchange rates from a configured retrieveal module on a periodic or on demand basis. By default exchange rates are retrieved from Open Exchange Rates . The default period of execution is 5 minutes (300_000 milliseconds). The period of retrieval is configured in config.exs or the appropriate environment configuration. For example: config :ex_money , retrieve_every : 300_000","ref":"Money.ExchangeRates.Retriever.html","title":"Money.ExchangeRates.Retriever","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Money.ExchangeRates.Retriever.html#child_spec/1","title":"Money.ExchangeRates.Retriever.child_spec/1","type":"function"},{"doc":"Return the current configuration of the Exchange Rates Retrieval service","ref":"Money.ExchangeRates.Retriever.html#config/0","title":"Money.ExchangeRates.Retriever.config/0","type":"function"},{"doc":"Delete the exchange rates retrieval service The service can be started again with start/1","ref":"Money.ExchangeRates.Retriever.html#delete/0","title":"Money.ExchangeRates.Retriever.delete/0","type":"function"},{"doc":"Forces retrieval of historic exchange rates for a single date date is a date returned by Date.new/3 or any struct with the elements :year , :month and :day or a Date.Range.t created by Date.range/2 that specifies a range of dates to retrieve Returns: {:ok, rates} if exchange rates request is successfully sent. {:error, reason} if the request cannot be send. Sends a message ot the exchange rate retrieval worker to request historic rates for a specified date or range be retrieved and stored. This function does not return exchange rates, for that see Money.ExchangeRates.latest_rates/0 or Money.ExchangeRates.historic_rates/1 .","ref":"Money.ExchangeRates.Retriever.html#historic_rates/1","title":"Money.ExchangeRates.Retriever.historic_rates/1","type":"function"},{"doc":"Forces retrieval of historic exchange rates for a range of dates from is a date returned by Date.new/3 or any struct with the elements :year , :month and :day . to is a date returned by Date.new/3 or any struct with the elements :year , :month and :day . Returns: {:ok, rates} if exchange rates request is successfully sent. {:error, reason} if the request cannot be send. Sends a message to the exchange rate retrieval process for each date in the range from .. to to request historic rates be retrieved.","ref":"Money.ExchangeRates.Retriever.html#historic_rates/2","title":"Money.ExchangeRates.Retriever.historic_rates/2","type":"function"},{"doc":"Forces retrieval of the latest exchange rates Sends a message ot the exchange rate retrieval worker to request current rates be retrieved and stored. Returns: {:ok, rates} if exchange rates request is successfully sent. {:error, reason} if the request cannot be send. This function does not return exchange rates, for that see Money.ExchangeRates.latest_rates/0 or Money.ExchangeRates.historic_rates/1 .","ref":"Money.ExchangeRates.Retriever.html#latest_rates/0","title":"Money.ExchangeRates.Retriever.latest_rates/0","type":"function"},{"doc":"Updated the configuration for the Exchange Rate Service","ref":"Money.ExchangeRates.Retriever.html#reconfigure/1","title":"Money.ExchangeRates.Retriever.reconfigure/1","type":"function"},{"doc":"Restart the exchange rates retrieval service","ref":"Money.ExchangeRates.Retriever.html#restart/0","title":"Money.ExchangeRates.Retriever.restart/0","type":"function"},{"doc":"Retrieve exchange rates from an external HTTP service. This function is primarily intended for use by an exchange rates api module.","ref":"Money.ExchangeRates.Retriever.html#retrieve_rates/2","title":"Money.ExchangeRates.Retriever.retrieve_rates/2","type":"function"},{"doc":"Starts the exchange rates retrieval service","ref":"Money.ExchangeRates.Retriever.html#start/1","title":"Money.ExchangeRates.Retriever.start/1","type":"function"},{"doc":"Stop the exchange rates retrieval service. The service can be restarted with restart/0 .","ref":"Money.ExchangeRates.Retriever.html#stop/0","title":"Money.ExchangeRates.Retriever.stop/0","type":"function"},{"doc":"Functions to manage the starting, stopping, deleting and restarting of the Exchange Rates Retriever.","ref":"Money.ExchangeRates.Supervisor.html","title":"Money.ExchangeRates.Supervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Money.ExchangeRates.Supervisor.html#child_spec/1","title":"Money.ExchangeRates.Supervisor.child_spec/1","type":"function"},{"doc":"Returns the name of the default supervisor which is Money.Supervisor","ref":"Money.ExchangeRates.Supervisor.html#default_supervisor/0","title":"Money.ExchangeRates.Supervisor.default_supervisor/0","type":"function"},{"doc":"Deleted the retriever child specification from the exchange rates supervisor. This is primarily of use if you want to change the configuration of the retriever after it is stopped and before it is restarted. In this situation the sequence of calls would be: iex&gt; Money.ExchangeRates.Retriever . stop iex&gt; Money.ExchangeRates.Retriever . delete iex&gt; Money.ExchangeRates.Retriever . start ( config )","ref":"Money.ExchangeRates.Supervisor.html#delete_retriever/0","title":"Money.ExchangeRates.Supervisor.delete_retriever/0","type":"function"},{"doc":"Restarts a stopped retriever. See also Money.ExchangeRates.Retriever.stop/0","ref":"Money.ExchangeRates.Supervisor.html#restart_retriever/0","title":"Money.ExchangeRates.Supervisor.restart_retriever/0","type":"function"},{"doc":"Returns a boolean indicating of the retriever process is configured and running","ref":"Money.ExchangeRates.Supervisor.html#retriever_running?/0","title":"Money.ExchangeRates.Supervisor.retriever_running?/0","type":"function"},{"doc":"Returns the status of the exchange rates retriever. The returned value is one of: :running if the service is running. In this state the valid action is Money.ExchangeRates.Service.stop/0 :stopped if it is stopped. In this state the valid actions are Money.ExchangeRates.Supervisor.restart_retriever/0 or Money.ExchangeRates.Supervisor.delete_retriever/0 :not_started if it is not configured in the supervisor and is not running. In this state the only valid action is Money.ExchangeRates.Supervisor.start_retriever/1","ref":"Money.ExchangeRates.Supervisor.html#retriever_status/0","title":"Money.ExchangeRates.Supervisor.retriever_status/0","type":"function"},{"doc":"Starts the Exchange Rates supervisor and optionally starts the exchange rates retrieval service as well. Options :restart is a boolean value indicating if the supervisor is to be restarted. This is typically used to move the supervisor from its default position under the ex_money supervision tree to a different supervision tree. The default is false :start_retriever is a boolean indicating if the exchange rates retriever is to be started when the supervisor is started. The default is defined by the configuration key :auto_start_exchange_rate_service","ref":"Money.ExchangeRates.Supervisor.html#start_link/0","title":"Money.ExchangeRates.Supervisor.start_link/0","type":"function"},{"doc":"","ref":"Money.ExchangeRates.Supervisor.html#start_link/1","title":"Money.ExchangeRates.Supervisor.start_link/1","type":"function"},{"doc":"Starts the exchange rates retriever Arguments config is a %Money.ExchangeRages.Config{} struct returned by Money.ExchangeRates.config/0 and adjusted as required. The default is Money.ExchangeRates.config/0","ref":"Money.ExchangeRates.Supervisor.html#start_retriever/1","title":"Money.ExchangeRates.Supervisor.start_retriever/1","type":"function"},{"doc":"Stop the Money.ExchangeRates.Supervisor. Unless ex_money is configured in mix.exs as rumtime: false , the Money.ExchangeRates.Supervisor is always started when ex_money starts even if the config key :auto_start_exchange_rates_service is set to false . In some instances an application may require the Money.ExchangeRates.Supervisor to be started under a different supervision tree. In this case it is required to call this function first before a new configuration is started. One use case is when the Exchange Rates service is configured with either an API module, a Callback module or a Cache module which uses Ecto and therefore its a requirement that Ecto is started first. See the README section on &quot;Using Ecto or other applications from within the callback module&quot; for an eanple of how to configure the supervisor in this case.","ref":"Money.ExchangeRates.Supervisor.html#stop/1","title":"Money.ExchangeRates.Supervisor.stop/1","type":"function"},{"doc":"Stop the exchange rates retriever.","ref":"Money.ExchangeRates.Supervisor.html#stop_retriever/0","title":"Money.ExchangeRates.Supervisor.stop_retriever/0","type":"function"},{"doc":"Provides functions to create, upgrade and downgrade subscriptions from one plan to another. Since moving from one plan to another may require prorating the payment stream at the point of transition, this module is introduced to provide a single point of calculation of the proration in order to give clear focus to the issues of calculating the carry-over amount or the carry-over period at the point of plan change. Defining a subscription A subscription records this current state and history of all plans assigned over time to a subscriber. The definition is deliberately minimal to simplify integration into applications that have a specific implementation of a subscription. A new subscription is created with Money.Subscription.new/3 which has the following attributes: plan which defines the initial plan for the subscription. This option is required. effective_date which determines the effective date of the inital plan. This option is required. options which include :created_at and :id with which a subscription may be annotated Changing a subscription plan Changing a subscription plan requires the following information be provided: A Subscription or the definition of the current plan The definition of the new plan The strategy for changing the plan which is either: to have the effective date of the new plan be after the current interval of the current plan To change the plan immediately in which case there will be a credit on the current plan which needs to be applied to the new plan. See Money.Subscription.change_plan/3 When the new plan is effective at the end of the current billing period The first strategy simply finishes the current billing period before the new plan is introduced and therefore no proration is required. This is the default strategy. When the new plan is effective immediately If the new plan is to be effective immediately then any credit balance remaining on the old plan needs to be applied to the new plan. There are two options of applying the credit: Reduce the billing amount of the first period of the new plan be the amount of the credit left on the old plan. This means that the billing amount for the first period of the new plan will be different (less) than the billing amount for subsequent periods on the new plan. Extend the first period of the new plan by the interval amount that can be funded by the credit amount left on the old plan. In the situation where the credit amount does not fully fund an integral interval the additional interval can be truncated or rounded up to the next integral period. Plan definition This module, and Money in general, does not provide a full billing or subscription solution - its focus is to support a reliable means of calcuating the accounting outcome of a plan change only. Therefore the plan definition required by Money.Subscription can be any Map.t that includes the following fields: interval which defines the time interval for a plan. The value can be one of day , week , month or year . interval_count which defines the number of interval s for the current plan interval. This must be a positive integer. price which is a Money.t representing the price of the plan to be paid each interval count. Billing in advance This module calculates all subscription changes on the basis that billing is done in advance. This primarily affects the calculation of plan credit when a plan changes. The assumption is that the period from the start of the current interval to the point of change has been consumed and therefore the credit is based upon that period of the plan that has not yet been consumed. If the calculation was based upon &quot;payment in arrears&quot; then the credit would actually be a debit since the part of the current period consumed has not yet been paid.","ref":"Money.Subscription.html","title":"Money.Subscription","type":"module"},{"doc":"A struct defining a subscription :id any term that uniquely identifies this subscription :plans is a list of {change, plan} tuples that record the history of plans assigned to this subscription :created_at records the DateTime.t when the subscription was created","ref":"Money.Subscription.html#__struct__/0","title":"Money.Subscription.__struct__/0","type":"function"},{"doc":"Cancel a subscription's pending plan. A pending plan arise when a a Subscription.change_plan/3 has been executed but the effective date is in the future. Only one plan may be pending at any one time so that if Subscription.change_plan/3 is attemtped a second time an error tuple will be returned. Subscription.cancel_pending_plan/2 can be used to roll back the pending plan change. Arguments :subscription is any Money.Subscription.t :options is a Keyword.t Options :today is a Date.t that represents today. The default is Date.utc_today Returns An updated Money.Subscription.t which may or may not have had a pending plan. If it did have a pending plan that plan is deleted. If there was no pending plan then the subscription is returned unchanged.","ref":"Money.Subscription.html#cancel_pending_plan/2","title":"Money.Subscription.cancel_pending_plan/2","type":"function"},{"doc":"Change plan from the current plan to a new plan. Arguments subscription_or_plan is either a Money.Subscription.t or Money.Subscription.Plan.t or a map with the same fields new_plan is a Money.Subscription.Plan.t or a map with at least the fields interval , interval_count and price current_interval_started is a Date.t or other map with the fields year , month , day and calendar options is a keyword list of options the define how the change is to be made Options :effective defines when the new plan comes into effect. The values are :immediately , a Date.t or :next_period . The default is :next_period . Note that the date applied in the case of :immediately is the date returned by Date.utc_today . :prorate which determines how to prorate the current plan into the new plan. The options are :price which will reduce the price of the first period of the new plan by the credit amount left on the old plan (this is the default). Or :period in which case the first period of the new plan is extended by the interval amount of the new plan that the credit on the old plan will fund. :round determines whether when prorating the :period it is truncated or rounded up to the next nearest full interval_count . Valid values are :down , :half_up , :half_even , :ceiling , :floor , :half_down , :up . The default is :up . :first_interval_started determines the anchor day for monthly billing. For example if a monthly plan starts on January 31st then the next period will start on February 28th (or 29th). The period following that should, however, be March 31st. If subscription_or_plan is a Money.Subscription.t then the :first_interval_started is automatically populated from the subscription. If :first_interval_started is nil then the date defined by :effective is used. Returns A Money.Subscription.Change.t with the following elements: :first_interval_starts which is the start date of the first interval for the new plan :first_billing_amount is the amount to be billed, net of any credit, at the :first_interval_starts :next_interval_starts is the start date of the next interval after the first interval including any credit_days_applied :credit_amount is the amount of unconsumed credit of the current plan :credit_amount_applied is the amount of credit applied to the new plan. If the :prorate option is :price (the default) then :first_billing_amount is the plan :price reduced by the :credit_amount_applied . If the :prorate option is :period then the :first_billing_amount is the plan price and the :next_interval_date is extended by the :credit_days_applied instead. :credit_days_applied is the number of days credit applied to the first interval by adding days to the :first_interval_starts date. :credit_period_ends is the date on which any applied credit is consumed or nil :carry_forward is any amount of credit carried forward to a subsequent period. If non-zero, this amount is a negative Money.t . It is non-zero when the credit amount for the current plan is greater than the :price of the new plan. In this case the :first_billing_amount is zero. Returns {:ok, updated_subscription} or {:error, {exception, message}} Examples # Change at end of the current period so no proration iex&gt; current = Money.Subscription.Plan . new! ( Money . new ( :USD , 10 ) , :month , 1 ) iex&gt; new = Money.Subscription.Plan . new! ( Money . new ( :USD , 10 ) , :month , 3 ) iex&gt; Money.Subscription . change_plan current , new , current_interval_started : ~D[2018-01-01] { :ok , % Money.Subscription.Change { carry_forward : Money . zero ( :USD ) , credit_amount : Money . zero ( :USD ) , credit_amount_applied : Money . zero ( :USD ) , credit_days_applied : 0 , credit_period_ends : nil , next_interval_starts : ~D[2018-05-01] , first_billing_amount : Money . new ( :USD , 10 ) , first_interval_starts : ~D[2018-02-01] } } # Change during the current plan generates a credit amount iex&gt; current = Money.Subscription.Plan . new! ( Money . new ( :USD , 10 ) , :month , 1 ) iex&gt; new = Money.Subscription.Plan . new! ( Money . new ( :USD , 10 ) , :month , 3 ) iex&gt; Money.Subscription . change_plan current , new , current_interval_started : ~D[2018-01-01] , effective : ~D[2018-01-15] { :ok , % Money.Subscription.Change { carry_forward : Money . zero ( :USD ) , credit_amount : Money . new ( :USD , &quot;5.49&quot; ) , credit_amount_applied : Money . new ( :USD , &quot;5.49&quot; ) , credit_days_applied : 0 , credit_period_ends : nil , next_interval_starts : ~D[2018-04-15] , first_billing_amount : Money . new ( :USD , &quot;4.51&quot; ) , first_interval_starts : ~D[2018-01-15] } } # Change during the current plan generates a credit period iex&gt; current = Money.Subscription.Plan . new! ( Money . new ( :USD , 10 ) , :month , 1 ) iex&gt; new = Money.Subscription.Plan . new! ( Money . new ( :USD , 10 ) , :month , 3 ) iex&gt; Money.Subscription . change_plan current , new , current_interval_started : ~D[2018-01-01] , effective : ~D[2018-01-15] , prorate : :period { :ok , % Money.Subscription.Change { carry_forward : Money . zero ( :USD ) , credit_amount : Money . new ( :USD , &quot;5.49&quot; ) , credit_amount_applied : Money . zero ( :USD ) , credit_days_applied : 50 , credit_period_ends : ~D[2018-03-05] , next_interval_starts : ~D[2018-06-04] , first_billing_amount : Money . new ( :USD , 10 ) , first_interval_starts : ~D[2018-01-15] } }","ref":"Money.Subscription.html#change_plan/3","title":"Money.Subscription.change_plan/3","type":"function"},{"doc":"Change plan from the current plan to a new plan. Retuns the plan or raises an exception on error. See Money.Subscription.change_plan/3 for the description of arguments, options and return.","ref":"Money.Subscription.html#change_plan!/3","title":"Money.Subscription.change_plan!/3","type":"function"},{"doc":"Returns the first date of the current interval of a plan. Arguments :subscription_or_changeset is any Money.Subscription.t or a {Change.t, Plan.t} tuple :options is a keyword list of options Options :today is a Date.t that represents today. The default is Date.utc_today Returns The Date.t that is the first date of the current interval","ref":"Money.Subscription.html#current_interval_start_date/2","title":"Money.Subscription.current_interval_start_date/2","type":"function"},{"doc":"Retrieve the plan that is currently in affect. The plan in affect is not necessarily the first plan in the list. We may have upgraded plans to be in affect at some later time. Arguments subscription is a Money.Subscription.t or any map that provides the field :plans Returns The Money.Subscription.Plan.t that is the plan currently in affect or nil","ref":"Money.Subscription.html#current_plan/2","title":"Money.Subscription.current_plan/2","type":"function"},{"doc":"Returns the start date of the current plan. Arguments subscription is a Money.Subscription.t or any map that provides the field :plans Returns The start Date.t of the current plan","ref":"Money.Subscription.html#current_plan_start_date/1","title":"Money.Subscription.current_plan_start_date/1","type":"function"},{"doc":"Returns number of days remaining in a plan interval. Arguments plan is any Money.Subscription.Plan.t current_interval_started is a Date.t effective_date is a Date.t after the current_interval_started and before the end of the plan_days Returns The number of days remaining in a plan interval Examples iex&gt; plan = Money.Subscription.Plan . new! Money . new! ( :USD , 100 ) , :month , 1 iex&gt; Money.Subscription . days_remaining plan , ~D[2018-01-01] , ~D[2018-01-02] 30 iex&gt; Money.Subscription . days_remaining plan , ~D[2018-02-01] , ~D[2018-02-02] 27","ref":"Money.Subscription.html#days_remaining/4","title":"Money.Subscription.days_remaining/4","type":"function"},{"doc":"Returns the latest plan for a subscription. The latest plan may not be in affect since its start date may be in the future. Arguments subscription is a Money.Subscription.t or any map that provides the field :plans Returns The Money.Subscription.Plan.t that is the most recent plan - whether or not it is the currently active plan.","ref":"Money.Subscription.html#latest_plan/1","title":"Money.Subscription.latest_plan/1","type":"function"},{"doc":"Creates a new subscription. Arguments plan is any Money.Subscription.Plan.t the defines the initial plan effective_date is a Date.t that represents the effective date of the initial plan. This defines the start of the first interval options is a keyword list of options Options :id is any term that an application can use to uniquely identify this subscription. It is not used in any function in this module. :created_at is a DateTime.t that records the timestamp when the subscription was created. The default is DateTime.utc_now/0 Returns {:ok, Money.Subscription.t} or {:error, {exception, message}}","ref":"Money.Subscription.html#new/3","title":"Money.Subscription.new/3","type":"function"},{"doc":"Creates a new subscription or raises an exception. Arguments plan is any Money.Subscription.Plan.t the defines the initial plan effective_date is a Date.t that represents the effective date of the initial plan. This defines the start of the first interval :options is a keyword list of options Options :id is any term that an application can use to uniquely identify this subscription. It is not used in any function in this module. :created_at is a DateTime.t that records the timestamp when the subscription was created. The default is DateTime.utc_now/0 Returns A Money.Subscription.t or raises an exception","ref":"Money.Subscription.html#new!/3","title":"Money.Subscription.new!/3","type":"function"},{"doc":"Returns the next interval start date for a plan. Arguments plan is any Money.Subscription.Plan.t :current_interval_started is the Date.t that represents the start of the current interval Returns The next interval start date as a Date.t . Example iex&gt; plan = Money.Subscription.Plan . new! ( Money . new! ( :USD , 100 ) , :month ) iex&gt; Money.Subscription . next_interval_starts ( plan , ~D[2018-03-01] ) ~D[2018-04-01] iex&gt; plan = Money.Subscription.Plan . new! ( Money . new! ( :USD , 100 ) , :day , 30 ) iex&gt; Money.Subscription . next_interval_starts ( plan , ~D[2018-02-01] ) ~D[2018-03-03]","ref":"Money.Subscription.html#next_interval_starts/3","title":"Money.Subscription.next_interval_starts/3","type":"function"},{"doc":"Returns number of days in a plan interval. Arguments plan is any Money.Subscription.Plan.t current_interval_started is any Date.t Returns The number of days in a plan interval. Examples iex&gt; plan = Money.Subscription.Plan . new! Money . new! ( :USD , 100 ) , :month , 1 iex&gt; Money.Subscription . plan_days plan , ~D[2018-01-01] 31 iex&gt; Money.Subscription . plan_days plan , ~D[2018-02-01] 28 iex&gt; Money.Subscription . plan_days plan , ~D[2018-04-01] 30","ref":"Money.Subscription.html#plan_days/3","title":"Money.Subscription.plan_days/3","type":"function"},{"doc":"Returns a boolean indicating if there is a pending plan. A pending plan is one where the subscription has changed plans but the plan is not yet in effect. There can only be one pending plan. Arguments :subscription is any Money.Subscription.t :options is a keyword list of options Options :today is a Date.t that represents the effective date used to determine is there is a pending plan. The default is Date.utc_today/1 . Returns Either true or false","ref":"Money.Subscription.html#plan_pending?/2","title":"Money.Subscription.plan_pending?/2","type":"function"},{"doc":"An id that uniquely identifies a subscription","ref":"Money.Subscription.html#t:id/0","title":"Money.Subscription.id/0","type":"type"},{"doc":"A Money.Subscription type","ref":"Money.Subscription.html#t:t/0","title":"Money.Subscription.t/0","type":"type"},{"doc":"Defines the structure of a plan changeset. :first_interval_starts which is the start date of the first interval for the new plan :first_billing_amount is the amount to be billed, net of any credit, at the :first_interval_starts :next_interval_starts is the start date of the next interval after the first interval including any credit_days_applied * :credit_amount is the amount of unconsumed credit of the current plan * :credit_amount_applied is the amount of credit applied to the new plan. If the :prorate option is :price (the default) the :first_billing_amount is the plan :price reduced by the :credit_amount_applied . If the :prorate option is :period then the :first_billing_amount is the plan price and the :next_interval_date is extended by the :credit_days_applied instead. :credit_days_applied is the number of days credit applied to the first interval by adding days to the :first_interval_starts date. :credit_period_ends is the date on which any applied credit is consumed or nil :carry_forward is any amount of credit carried forward to a subsequent period. If non-zero this amount is a negative Money.t . It is non-zero when the credit amount for the current plan is greater than the price of the new plan. In this case the :first_billing_amount is zero.","ref":"Money.Subscription.Change.html","title":"Money.Subscription.Change","type":"module"},{"doc":"A struct defining the changes between two plans.","ref":"Money.Subscription.Change.html#__struct__/0","title":"Money.Subscription.Change.__struct__/0","type":"function"},{"doc":"A plan change record struct.","ref":"Money.Subscription.Change.html#t:t/0","title":"Money.Subscription.Change.t/0","type":"type"},{"doc":"","ref":"Money.Subscription.DateError.html","title":"Money.Subscription.DateError","type":"exception"},{"doc":"","ref":"Money.Subscription.NoCurrentPlan.html","title":"Money.Subscription.NoCurrentPlan","type":"exception"},{"doc":"Defines a standard subscription plan data structure.","ref":"Money.Subscription.Plan.html","title":"Money.Subscription.Plan","type":"module"},{"doc":"Defines the structure of a subscription plan.","ref":"Money.Subscription.Plan.html#__struct__/0","title":"Money.Subscription.Plan.__struct__/0","type":"function"},{"doc":"Returns {:ok, Money.Subscription.Plan.t} or an {:error, reason} tuple. Arguments :price is any Money.t :interval is the period of the plan. The valid intervals are :day , :week , :month or ':year . :interval_count is an integer count of the number of :interval s of the plan. The default is 1 Returns A Money.Subscription.Plan.t Examples iex&gt; Money.Subscription.Plan . new Money . new ( :USD , 100 ) , :month , 1 { :ok , % Money.Subscription.Plan { interval : :month , interval_count : 1 , price : Money . new ( :USD , 100 ) } } iex&gt; Money.Subscription.Plan . new Money . new ( :USD , 100 ) , :month { :ok , % Money.Subscription.Plan { interval : :month , interval_count : 1 , price : Money . new ( :USD , 100 ) } } iex&gt; Money.Subscription.Plan . new Money . new ( :USD , 100 ) , :day , 30 { :ok , % Money.Subscription.Plan { interval : :day , interval_count : 30 , price : Money . new ( :USD , 100 ) } } iex&gt; Money.Subscription.Plan . new 23 , :day , 30 { :error , { Money.Invalid , &quot;Invalid subscription plan definition&quot; } }","ref":"Money.Subscription.Plan.html#new/3","title":"Money.Subscription.Plan.new/3","type":"function"},{"doc":"Returns {:ok, Money.Subscription.Plan.t} or raises an exception. Takes the same arguments as Money.Subscription.Plan.new/3 . Example iex&gt; Money.Subscription.Plan . new! Money . new ( :USD , 100 ) , :day , 30 % Money.Subscription.Plan { interval : :day , interval_count : 30 , price : Money . new ( :USD , 100 ) }","ref":"Money.Subscription.Plan.html#new!/3","title":"Money.Subscription.Plan.new!/3","type":"function"},{"doc":"A plan interval type.","ref":"Money.Subscription.Plan.html#t:interval/0","title":"Money.Subscription.Plan.interval/0","type":"type"},{"doc":"A integer interval count for a plan.","ref":"Money.Subscription.Plan.html#t:interval_count/0","title":"Money.Subscription.Plan.interval_count/0","type":"type"},{"doc":"A Subscription Plan","ref":"Money.Subscription.Plan.html#t:t/0","title":"Money.Subscription.Plan.t/0","type":"type"},{"doc":"","ref":"Money.Subscription.PlanError.html","title":"Money.Subscription.PlanError","type":"exception"},{"doc":"","ref":"Money.Subscription.PlanPending.html","title":"Money.Subscription.PlanPending","type":"exception"},{"doc":"","ref":"Money.ExchangeRateError.html","title":"Money.ExchangeRateError","type":"exception"},{"doc":"","ref":"Money.Invalid.html","title":"Money.Invalid","type":"exception"},{"doc":"","ref":"Money.InvalidAmountError.html","title":"Money.InvalidAmountError","type":"exception"},{"doc":"","ref":"Money.InvalidDigitsError.html","title":"Money.InvalidDigitsError","type":"exception"},{"doc":"","ref":"Money.ParseError.html","title":"Money.ParseError","type":"exception"},{"doc":"","ref":"Money.UnknownCurrencyError.html","title":"Money.UnknownCurrencyError","type":"exception"},{"doc":"Money implements a set of functions to store, retrieve, convert and perform arithmetic on a Money.t/0 type that is composed of an ISO 4217 currency code or an ISO 24165 Digital Token Identifier (crypto currency) with a currency amount. Money is opinionated in the interests of serving as a dependable library that can underpin accounting and financial applications. How is this opinion expressed? Money must always have both a amount and a currency code or digital token identifier. The currency code must always be a valid ISO 4217 code or a valid ISO 24165 digital token idenfier. Current and historical currency codes can be used. See the ISO Currency for more information. You can also identify the relevant codes by: Money.known_currencies/0 returns all the ISO 4217 currency codes known to Money Money.known_current_currencies/0 returns the ISO 4217 currency codes currently in use Money.known_historic_currencies/0 returns the list of historic ISO 4217 currency codes Money.known_tender_currencies/0 returns the list of ISO 4217 currencies known to be legal tender DigitalToken.tokens/0 returns a map of the known ISO 24165 digital tokens. Money arithmetic can only be performed when both operands are of the same currency. Money amounts are represented as a Decimal . Money can be serialised to the database as a composite Postgres type that includes both the amount and the currency. For MySQL, money is serialized into a json column with the amount converted to a string to preserve precision since json does not have a decimal type. Serialization is entirely optional. All arithmetic functions work on a Decimal . No rounding occurs automatically (unless expressly called out for a function, as is the case for Money.split/2 ). Explicit rounding obeys the rounding rules for a given currency. The rounding rules are defined by the Unicode consortium in its CLDR repository as implemented by the hex package ex_cldr . These rules define the number of fractional digits for a currency and the rounding increment where appropriate. Money output string formatting output using the hex package ex_cldr that correctly rounds to the appropriate number of fractional digits and to the correct rounding increment for currencies that have minimum cash increments (like the Swiss Franc and Australian Dollar)","ref":"readme.html","title":"Introduction to Money","type":"extras"},{"doc":"Money is supported on Elixir 1.10 and later only.","ref":"readme.html#prerequisities","title":"Introduction to Money - Prerequisities","type":"extras"},{"doc":"Money starts a supervisor Money.Supervisor by default unless the dependency is configured as runtime: false in mix.exs . If configured as runtime: false then the application can be started later via Money.Application.start(:normal, supervisor_options) where supervisor_options is a keyword list of options that is given the Supervisor.start_link/2 . The default options are [strategy: :one_for_one, name: Money.Supervisor] . The application supervisor is used by default to start the exchange rates service when required. The exchange rate service can be configured to run in a user defined supervision tree as explained in the next section.","ref":"readme.html#supervisor-configuration-and-operation","title":"Introduction to Money - Supervisor configuration and operation","type":"extras"},{"doc":"As of ex_cldr_currencies version 2.6.0 it is possible to define private use currencies. These are currencies that are ISO 4217 compliant but guaranteed never to be allocated by the ISO committee and therefore safe to be used by developers. Defining private use currencies See Cldr.Currency.new/2 Starting the private use currency store In order to define private use currencies, a special :ets table is required to hold their definitions. The is implemented by a supervisor and two workers that together aim to preserve the availability of the :ets table as resiliently as possible. The implementation is an embedded and updated version of eternal . The basic requirement is to add a the private use currency supervisor to your applications supervision tree. For example: defmodule MyApp do use Application def start ( _type , _args ) do # Start the service which maintains the # :ets table that holds the private use currencies children = [ Cldr.Currency ... ] opts = [ strategy : :one_for_one , name : MoneyTest.Supervisor ] Supervisor . start_link ( children , opts ) end end It is also possible to define a callback that is called when the Cldr.Currency supervisor is started so that private use currencies can be defined. For further information see Defining Private Use Currencies .","ref":"readme.html#private-use-currencies","title":"Introduction to Money - Private Use Currencies","type":"extras"},{"doc":"Money includes a process to retrieve exchange rates on a periodic basis. These exchange rates can then be used to support currency conversion. This service is not started by default. If started it will attempt to retrieve exchange rates every 5 minutes by default. By default, exchange rates are retrieved from Open Exchange Rates however any module that conforms to the Money.ExchangeRates behaviour can be configured. An optional callback module can also be defined. This module defines a rates_retrieved/2 function that is invoked upon every successful retrieval of exchange rates. This might be used to serialize exchange rate to a data store or to stream rates to other applications or systems.","ref":"readme.html#exchange-rates-and-currency-conversion","title":"Introduction to Money - Exchange rates and currency conversion","type":"extras"},{"doc":"Money provides a set of configuration keys to customize behaviour. The default configuration is: config :ex_money , exchange_rates_retrieve_every : 300_000 , api_module : Money.ExchangeRates.OpenExchangeRates , callback_module : Money.ExchangeRates.Callback , exchange_rates_cache_module : Money.ExchangeRates.Cache.Ets , preload_historic_rates : nil , retriever_options : nil , log_failure : :warn , log_info : :info , log_success : nil , json_library : Jason , default_cldr_backend : MyApp.Cldr , exclude_protocol_implementations : [ ] Configuration key definitions :default_cldr_backend defines the Cldr backend module that is default for Money . See the ex_cldr documentation for further information on how to define this module. This is a required option . :exchange_rates_retrieve_every defines how often the exchange rates are retrieved in milliseconds. The default is :never . An atom value is interpreted to mean that there should be no periodic retrieval. :api_module identifies the module that does the retrieval of exchange rates. This is any module that implements the Money.ExchangeRates behaviour. The default is Money.ExchangeRates.OpenExchangeRates . :exchange_rates_cache_module defines the module that provides an exchange rates cache. Any module that implements the Money.ExchangeRates.Cache behaviour. Two alternative strategies are provided: Money.ExchangeRates.Cache.Ets which is also the default. Money.ExchangeRates.Cache.Dets :preload_historic_rates defines a date or a date range that will be requested when the exchange rate service starts up. The date or date range should be specified as either a Date.t or a Date.Range.t or a tuple of {Date.t, Date.t} representing the from and to dates for the rates to be retrieved. The default is nil meaning no historic rates are preloaded. See Preloading historic rates for more information. callback_module defines a module that follows the Money.ExchangeRates.Callback behaviour whereby the function rates_retrieved/2 is invoked after every successful retrieval of exchange rates. The default is Money.ExchangeRates.Callback . log_failure defines the log level at which api retrieval errors are logged. The default is :warn . log_success defines the log level at which successful api retrieval notifications are logged. The default is nil which means no logging. log_info defines the log level at which service startup messages are logged. The default is info . :retriever_options is available for exchange rate retriever module developers as a place to add retriever-specific configuration information. This information should be added in the init/1 callback in the retriever module. See Money.ExchangeRates.OpenExchangeRates.init/1 for an example. :json_library determines which json library to be used for decoding. Two common options are Poison and Jason . The default is Cldr.Config.json_library/0 which is currently configured by default as Jason . :exclude_protocol_implementations is a protocol module, or list of protocol modules, that will not be defined by ex_money . The default is [] . The protocol implementations influenced by this option at Json.Encoder , Phoenix.HTML.Safe and Gringotts.Money . JSON library configuration Note that ex_money does not define a json library dependency and therefore it is the users responsibility to configure the required json library as a dependency in the application's mix.exs . The recommended library is jason which would be configured as: defp deps do [ { :jason , &quot;~&gt; 1.0&quot; } , ... ] end ex_money depends on ex_cldr which provides currency and localisation data. The default configuration of ex_money uses the default json_library from ex_cldr . This can be configured as follows in config.exs : config :ex_cldr , json_library : Jason In most cases this is not required since the presence of Jason (or Poison ) is automatic. Configuring locales to support localised formatting Money uses ex_cldr and ex_cldr_numbers to support configuring locales and providing locale formatting. These packages are also the source of currency definitions, names, formats and so on. To use Cldr and therefore Money , a backend module must be defined. This module will host the Cldr data and public API used by Money . A simple example would be: defmodule MyApp.Cldr do use Cldr , locales : [ &quot;en&quot; , &quot;fr&quot; , &quot;zh&quot; ] , default_locale : &quot;en&quot; , providers : [ Cldr.Number , Money ] end Preloading historic rates The current implementation will call the api_module to retrieve the historic rates once for each date in the :preload_historic_rates range. Some exchange rate services, like Open Exchange Rates, provides a bulk retrieval api that can retrieve multiple dates in a single call. However this endpoint is only available for premium subscribers and it is still charged on a &quot;per date retrieved&quot; basis. So while there is a network/performance/efficiency benefit there is no economic benefit. Please file an issue on github if implementing a bulk api is important to you. Some examples of configuring the :preload_historic_rates key follow: preload_historic_rates: ~D[2017-01-01] preload_historic_rates: Date.range(~D[2017-01-01], ~D[2017-10-01]) preload_historic_rates: {~D[2017-01-01], ~D[2017-10-01]} Open Exchange Rates configuration If you plan to use the provided Open Exchange Rates module to retrieve exchange rates then you should also provide the addition configuration key for app_id : config :ex_money , open_exchange_rates_app_id : &quot;your_app_id&quot; or configure it via environment variable, for example: config :ex_money , open_exchange_rates_app_id : { :system , &quot;OPEN_EXCHANGE_RATES_APP_ID&quot; } The default exchange rate retrieval module is provided in Money.ExchangeRates.OpenExchangeRates which can be used as a example to implement your own retrieval module for other services. Managing the configuration at runtime During exchange rate service startup, the function init/1 is called on the configured exchange rate retrieval module. This module is expected to return an updated configuration allowing a developer to customise how the configuration is to be managed. See the implementation at Money.ExchangeRates.OpenExchangeRates.init/1 for an example. To support runtime (re-)configuration the following functions are provided: Money.ExchangeRates.Retriever.config/0 returns the current configuration of the exchange rates retrieval service. Money.ExchangeRates.Retriever.stop/0 and Money.ExchangeRates.Retriever.start/0 stop and start the exchange rates retrieval service respectively. Money.ExchangeRates.Retriever.reconfigure/1 reconfigures the exchange rates retrieval service. It does not restart the service, the service remains active during the recongiguration. Using Environment Variables in the configuration Keys can also be configured to retrieve values from environment variables. This lookup is done at runtime to facilitate deployment strategies. If the value of a configuration key is {:system, &quot;some_string&quot;} then &quot;some_string&quot; is interpreted as an environment variable name which is passed to System.get_env/2 . An example configuration might be: config :ex_money , auto_start_exchange_rate_service : { :system , &quot;RATE_SERVICE&quot; } , exchange_rates_retrieve_every : { :system , &quot;RETRIEVE_EVERY&quot; } , open_exchange_rates_app_id : { :system , &quot;OPEN_EXCHANGE_RATES_APP_ID&quot; } Note that the {:system, &quot;ENV KEY&quot;} approach is not currently supported for the :preload_historic_rates configuration key.","ref":"readme.html#configuration","title":"Introduction to Money - Configuration","type":"extras"},{"doc":"If the exchange rate service is configured to automatically start up (because the config key auto_start_exchange_rate_service is set to true ) then a supervisor process named Money.ExchangeRates.Supervisor is started which in turns starts a child GenServer called Money.ExchangeRates.Retriever . It is Money.ExchangeRates.Retriever which will call the configured api_module to retrieve the rates. It is also responsible for calling the configured callback_module after a successfull retrieval. + -- -- -- -- -- -- -- -- - + | | + -- -- -- -- -- -- - + + -- -- -- -- -- - + | api_module | -&gt; External Service | | | | -- -&gt; | | | Supervisor | -- -&gt; | Retriever | + -- -- -- -- -- -- -- -- - + | | | | -- -&gt; + -- -- -- -- -- -- -- -- - + + -- -- -- -- -- -- - + + -- -- -- -- -- - + | | | callback_module | | | + -- -- -- -- -- -- -- -- - + On application start (or manual start if :auto_start_exchange_rate_service is set to false ), Money.ExchangeRates.Retriever will schedule the first retrieval to be executed after immediately and then each :exchange_rates_retrieve_every milliseconds thereafter.","ref":"readme.html#the-exchange-rates-service-process-supervision-and-startup","title":"Introduction to Money - The Exchange rates service process supervision and startup","type":"extras"},{"doc":"If you provide your own callback module and that module depends on some other applications, like Ecto , already being started then automatically starting Money.ExchangeRates.Supervisor may not work since your Ecto.Repo is unlikely to have already been started. In this situation the appropriate way to configure the exchange rates retrieval service is the following: Set the configuration key auto_start_exchange_rate_service to false to prevent automatic startup of the service. Configure your api_module , callback_module and any other required configuration as appropriate In your client application code, add the Money.ExchangeRates.Supervisor to the children configuration of your application. For example, in an application that uses Ecto and where your callback_module is designed to save exchange rates to a database, your application may would look something like: defmodule Application do use Application def start ( _type , _args ) do import Supervisor.Spec children = [ # Start your repo first so that it is running before your # exchange rates callback module is called supervisor ( MoneyTest.Repo , [ ] ) , # Include the Money.ExchangeRates.Supervisor in your application&#39;s # supervision tree. This supervisor will start the child process # Money.ExchangeRates.Retriever. # Note the use of double `[]` around # the parameters which are required to ensure that the supervisor # is stopped before including in your supervisor tree. # The `start_retriever: true` is optional. The default value is `false`. supervisor ( Money.ExchangeRates.Supervisor , [ [ restart : true , start_retriever : true ] ] ) ] opts = [ strategy : :one_for_one , name : Application.Supervisor ] Supervisor . start_link ( children , opts ) end end","ref":"readme.html#using-ecto-or-other-applications-from-within-the-callback-module","title":"Introduction to Money - Using Ecto or other applications from within the callback module","type":"extras"},{"doc":"Creating a %Money{} struct iex&gt; Money . new ( :USD , 100 ) # Money &lt; :USD , 100 &gt; iex&gt; Money . new ( 100 , :USD ) # Money &lt; :USD , 100 &gt; iex&gt; Money . new ( &quot;CHF&quot; , &quot;130.02&quot; ) # Money &lt; :CHF , 130.02 &gt; iex&gt; Money . new ( &quot;thb&quot; , 11 ) # Money &lt; :THB , 11 &gt; iex&gt; Money . new ( &quot;1.000,99&quot; , :EUR , locale : &quot;de&quot; ) # Money &lt; :EUR , 1000.99 &gt; iex&gt; Money . parse ( &quot;USD 100&quot; ) # Money &lt; :USD , 100 &gt; iex&gt; Money . parse ( &quot;USD 100,00&quot; , locale : &quot;de&quot; ) # Money &lt; :USD , 100.00 &gt; The canonical representation of a currency code is an atom that is a valid ISO4217 currency code or a String.t/0 representation of an ISO 24165 digital token identifier. The amount of a %Money{} is represented by a Decimal . Note that the amount and currency code arguments to Money.new/3 can be supplied in either order. Parsing money strings Money provides an ability to parse strings that contain a currency and an amount. The currency can be represented in different ways depending on the locale. See Money.parse/2 for further information. Some examples are: # These are the strings available for a given currency # and locale that are recognised during parsing iex&gt; Cldr.Currency . strings_for_currency :AUD , &quot;de&quot; [ &quot;aud&quot; , &quot;au$&quot; , &quot;australischer dollar&quot; , &quot;australische dollar&quot; ] iex&gt; Money . parse &quot;$au 12 346&quot; , locale : &quot;fr&quot; # Money &lt; :AUD , 12346 &gt; iex&gt; Money . parse &quot;12 346 dollar australien&quot; , locale : &quot;fr&quot; # Money &lt; :AUD , 12346 &gt; iex&gt; Money . parse &quot;A$ 12346&quot; , locale : &quot;en&quot; # Money &lt; :AUD , 12346 &gt; iex&gt; Money . parse &quot;australian dollar 12346.45&quot; , locale : &quot;en&quot; # Money &lt; :AUD , 12346.45 &gt; # Parse using a default currency iex&gt; Money . parse ( &quot;100&quot; , default_currency : :EUR ) # Money &lt; :EUR , 100 &gt; # Parse using the default currency of the locale # If no `:locale` option is provided then # the locale associated with `Money.default_backend/0` # is used. iex&gt; Money . parse ( &quot;100&quot; , locale : &quot;en&quot; ) # Money &lt; :EUR , 100 &gt; # Parse with a default currency for the current locale iex&gt; Money . parse ( &quot;100&quot; , default_currency : Money . default_currency_for_locale ( ) ) # Money &lt; :USD , 100 &gt; # Note that the decimal separator in the &quot;de&quot; locale # is a `.` iex&gt; Money . parse &quot;AU$ 12346,45&quot; , locale : &quot;de&quot; # Money &lt; :AUD , 12346.45 &gt; # Round trip formatting is supported iex&gt; { :ok , string } = Cldr.Number . to_string 1234 , Money.Cldr , currency : :AUD { :ok , &quot;A$1,234.00&quot; } iex&gt; Money . parse string # Money &lt; :AUD , 1234.00 &gt; # Fuzzy matching is possible iex&gt; Money . parse ( &quot;100 eurosports&quot; , fuzzy : 0.8 ) # Money &lt; :EUR , 100 &gt; iex&gt; Money . parse ( &quot;100 eurosports&quot; , fuzzy : 0.9 ) { :error , { Money.Invalid , &quot;Unable to create money from \\&quot; eurosports \\&quot; and \\&quot; 100 \\&quot; &quot; } } # Eligible currencies can be filtered iex&gt; Money . parse ( &quot;100 eurosports&quot; , fuzzy : 0.8 , only : [ :current ] ) # Money &lt; :EUR , 100 &gt; iex&gt; Money . parse ( &quot;100 euro&quot; , only : [ :EUR , :USD , :COP ] ) # Money &lt; :EUR , 100 &gt; iex&gt; Money . parse ( &quot;100 euro&quot; , except : [ :EUR , :USD , :COP ] ) { :error , { Money.UnknownCurrencyError , &quot;The currency \\&quot; euro \\&quot; is unknown or not supported&quot; } } iex&gt; Money . parse &quot;100 afghan afghanis&quot; # Money &lt; :AFN , 100 &gt; iex&gt; Money . parse &quot;100 afa&quot; , only : [ :current ] { :error , { Money.UnknownCurrencyError , &quot;The currency \\&quot; afa \\&quot; is unknown or not supported&quot; } } Casting a money type (basic support for HTML forms) Money supports form field inputs that are a single string combining both a currency code and an amount. When a form field (or other data) is cast then Money will attempt to parse a string field into a Money.t using Money.parse/2 . Therefore simple money form input can be supported with a single input field of type=text . Note that when parsing the input text, the amount is interpreted in the context of the current locale set on the default backend configured for ex_money . This affects how separator characters are interpreted in exactly the same way as is done for Money.new/3 . Float amounts cannot be provided to Money.new/2 Float have well-known issues in computing due to issues of rounding and potential precision loss. Internally Money uses Decimal to store the amount which allows arbitrary precision arithmetic. Money also uses the numeric type in Postgres to preserve precision and even goes to far as to store the amount as a string in MySQL for the same reason. Therefore an error is returned if an attempt is made to use Money.new/2 with a float amount: { :error , { Money.InvalidAmountError , &quot;Float amounts are not supported in new/2 due to potenial rounding &quot; &lt;&gt; &quot;and precision issues. If absolutely required, use Money.from_float/2&quot; } } If the use of float s is require then the function Money.from_float/2 is provided with the same arguments as those for Money.new/2 . Money.from_float/2 provides an addition check and will return an error if the precision (number of digits) of the provided float is more than 15 (the number of digits guaranteed to round-trip between a 64-bit float and a string). Comparison functions Money values can be compared as long as they have the same currency. The recommended function is Money.compare/2 which, given two compatible money amounts, will return :lt , :eq or :gt depending on the relationship. For example: iex&gt; Money . compare Money . new ( :USD , 100 ) , Money . new ( :USD , 200 ) :lt iex&gt; Money . compare Money . new ( :USD , 100 ) , Money . new ( :AUD , 200 ) { :error , { ArgumentError , &quot;Cannot compare monies with different currencies. Received :USD and :AUD.&quot; } } From Elixir verison 1.10.0 onwards, several functions in the Enum module can use the Money.compare/2 function to simplify sorting. For example: iex&gt; list = [ Money . new ( :USD , 100 ) , Money . new ( :USD , 200 ) ] [ # Money &lt; :USD , 100 &gt; , # Money &lt; :USD , 200 &gt; ] iex&gt; Enum . sort list , Money [ # Money &lt; :USD , 100 &gt; , # Money &lt; :USD , 200 &gt; ] iex&gt; Enum . sort list , { :asc , Money } [ # Money &lt; :USD , 100 &gt; , # Money &lt; :USD , 200 &gt; ] iex&gt; Enum . sort list , { :desc , Money } [ # Money &lt; :USD , 200 &gt; , # Money &lt; :USD , 100 &gt; ] Note that Enum.sort/2 will sort money amounts even when the currencies are incompatible. In this case the order of the result is not predictable. It is the developers responsibility to filter the list to compatible currencies prior to sorting. This is a limitation of the Enum.sort/2 implementation. Optional ~M sigil An optional sigil module is available to aid in creating %Money{} structs. It needs to be imported before use: import Money.Sigil ~M[100]USD #&gt; #Money&lt;:USD, 100&gt; Localised Money formatting Money provides locale-specific formatted output that is controlled be either the locale that has been set for this process or by the :locale parameter supplied to Money.to_string/2 . Configuring your localised environment requires configuring ex_cldr which is a dependency to Money . See the Configuration section of the ex_cldr readme for more information. The main API for formatting Money is Money.to_string/2 . Additionally formatting options are passed to Cldr.Number.to_string/2 . Those options are described in the readme for ex_cldr_numbers which is also a dependency to Money . iex&gt; Money . to_string Money . new ( &quot;thb&quot; , 11 ) { :ok , &quot;THB11.00&quot; } # The default locale is &quot;en-001&quot; which is # &quot;global english&quot; iex&gt; Money . to_string Money . new ( &quot;USD&quot; , &quot;234.467&quot; ) { :ok , &quot;$US234.47&quot; } # The locale &quot;en&quot; is &quot;American English&quot;. For # UK English use the locale &quot;en-GB&quot;. Australian # English is &quot;en-AU&quot; and so on. iex&gt; Money . to_string Money . new ( &quot;USD&quot; , &quot;234.467&quot; ) , locale : &quot;en&quot; { :ok , &quot;$234.47&quot; } iex&gt; Money . to_string Money . new ( &quot;USD&quot; , &quot;234.467&quot; ) , format : :long { :ok , &quot;234.47 US dollars&quot; } iex&gt; Money . to_string Money . new ( &quot;USD&quot; , &quot;234.467&quot; ) , locale : &quot;fr&quot; { :ok , &quot;234,47 $US&quot; } iex&gt; Money . to_string Money . new ( &quot;USD&quot; , &quot;234.467&quot; ) , locale : &quot;de&quot; { :ok , &quot;234,47 $&quot; } iex&gt; Money . to_string Money . new ( &quot;EUR&quot; , &quot;234.467&quot; ) , locale : &quot;de&quot; { :ok , &quot;234,47 €&quot; } iex&gt; Money . to_string Money . new ( &quot;EUR&quot; , &quot;234.467&quot; ) , locale : &quot;fr&quot; { :ok , &quot;234,47 €&quot; } Note that the output is influenced by the locale in effect. By default the locale used is that returned by Cldr.get_locale/0 . Its default value is :en-001 . Additional locales can be configured, see Cldr . The formatting options are defined in Cldr.Number.to_string/2 . Arithmetic Functions See also the module Money.Arithmetic : iex&gt; m1 = Money . new ( :USD , 100 ) # Money &lt; :USD , 100 &gt; } iex&gt; m2 = Money . new ( :USD , 200 ) # Money &lt; :USD , 200 &gt; } iex&gt; Money . add ( m1 , m2 ) { :ok , # Money &lt; :USD , 300 &gt; } iex&gt; Money . add! ( m1 , m2 ) # Money &lt; :USD , 300 &gt; iex&gt; m3 = Money . new ( :AUD , 300 ) # Money &lt; :AUD , 300 &gt; iex&gt; Money . add Money . new ( :USD , 200 ) , Money . new ( :AUD , 100 ) { :error , { ArgumentError , &quot;Cannot add monies with different currencies. Received :USD and :AUD.&quot; } } # Split a %Money{} returning the a dividend and a remainder. All # operations respect the number of fractional digits defined for a currency iex&gt; m1 = Money . new ( :USD , 100 ) # Money &lt; :USD , 100 &gt; iex&gt; Money . split ( m1 , 3 ) { # Money &lt; :USD , 33.33 &gt; , # Money &lt; :USD , 0.01 &gt; } # Rounding applies the currency definitions of CLDR as implemented in # the hex package [ex_cldr](https://hex.pm/packages/ex_cldr) iex&gt; Money . round Money . new ( :USD , &quot;100.678&quot; ) # Money &lt; :USD , 100.68 &gt; iex&gt; Money . round Money . new ( :JPY , &quot;100.678&quot; ) # Money &lt; :JPY , 101 &gt; Currency Conversion A %Money{} struct can be converted to another currency using Money.to_currency/3 or Money.to_currency!/3 . For example: iex&gt; Money . to_currency Money . new ( :USD , 100 ) , :AUD { :ok , # Money &lt; :AUD , 136.43 &gt; } iex&gt; Money . to_currency Money . new ( :USD , 100 ) , :AUD , ExchangeRates . historic_rates ( ~D[2017-01-01] ) { :ok , # Money &lt; :AUD , 128.76 &gt; } iex&gt; Money . to_currency Money . new ( :USD , 100 ) , :AUDD , %{ USD : Decimal . new ( 1 ) , AUD : Decimal . new ( 0.7345 ) } { :error , { Cldr.UnknownCurrencyError , &quot;Currency :AUDD is not known&quot; } } iex&gt; Money . to_currency! Money . new ( :USD , 100 ) , :XXX ** (Money.ExchangeRateError) No exchange rate is available for currency :XXX A user-defined map of exchange rates can also be supplied: iex&gt; Money . to_currency Money . new ( :USD , 100 ) , :AUD , %{ USD : Decimal . new ( 1.0 ) , AUD : Decimal . new ( 1.3 ) } # Money &lt; :AUD , 130 &gt; Historic Conversion Rates As noted in the configuration section, ex_money can preload historic exchange rates when the exchange rates service starts up. It can be anticipated that additional historic rates may be required subsequently. Money.ExchangeRates.Retriever.historic_rates/1 can be called to request retrieval of historic rates at any time. This call will send a message to the retrieval service to request retrieval. It does not return the rates. Money.ExchangeRates.historic_rates/1 is the partner function to Money.ExchangeRates.latest_rates/0 . It returns the exchange rates for a given date, and will return an error if no rates are available. Financial Functions A set of basic financial functions are available in the module Money.Financial . These functions are: Present value: Money.Financial.present_value/3 Future value: Money.Financial.future_value/3 Interest rate: Money.Financial.interest_rate/3 Number of periods: Money.Financial.periods/3 Payment amount: Money.Financial.payment/3 Net Present Value of a set of cash flows: Money.Financial.net_present_value/2 Internal rate of return: Money.Financial.internal_rate_of_return/1 For more detail see Money.Financial .","ref":"readme.html#api-usage-examples","title":"Introduction to Money - API Usage Examples","type":"extras"},{"doc":"Subscriptions, especially in the context of a SaaS, can involve changing plans - either from a smaller plan to a larger or a larger plan to smaller. In either situation a credit amount needs to be calculated based upon the current plan which is then applied to the new plan. Money.Subscription is a module that provides functions to support this subscription pricing, credit calculations and payment dates. The primary functions supporting subscriptions are: Create a new subscription: Money.Subscription.new/3 Create a subscription plan: Money.Subscription.Plan.new/3 Change a from one plan to another: Money.Subscription.change_plan/3 Calculate the start date for the next interval of a plan: Money.Subscription.next_interval_starts/3 Calculate the number of days in a plan interval: Money.Subscription.plan_days/3 Calculate the number of days left in a plan interval: Money.Subscription.days_remaining/4 Examples # Create the current plan iex&gt; current_plan = Money.Subscription.Plan . new! ( Money . new ( :USD , 10 ) , :month , 1 ) % Money.Subscription.Plan { interval : :month , interval_count : 1 , price : # Money &lt; :USD , 10 &gt; } # How many days in a billing period? iex&gt; Money.Subscription . plan_days current_plan , ~D[2018-03-01] 31 iex&gt; Money.Subscription . plan_days current_plan , ~D[2018-02-01] 28 # How many days remaining in the current billing period iex&gt; Money.Subscription . days_remaining current_plan , ~D[2018-03-01] , ~D[2018-03-10] 22 # When is the next billing date iex&gt; Money.Subscription . next_interval_starts current_plan , ~D[2018-03-01] ~D[2018-04-01] # Create a new plan iex&gt; new_plan = Money.Subscription.Plan . new! ( Money . new ( :USD , 10 ) , :month , 3 ) % Money.Subscription.Plan { interval : :month , interval_count : 3 , price : # Money &lt; :USD , 10 &gt; } # Change plans at the end of the current billing period iex&gt; Money.Subscription . change_plan current_plan , new_plan , current_interval_started : ~D[2018-03-01] % Money.Subscription.Change { carry_forward : # Money &lt; :USD , 0 &gt; , credit_amount : # Money &lt; :USD , 0 &gt; , credit_amount_applied : # Money &lt; :USD , 0 &gt; , credit_days_applied : 0 , credit_period_ends : nil , first_billing_amount : # Money &lt; :USD , 10 &gt; , first_interval_starts : ~D[2018-04-01] , next_interval_starts : ~D[2018-07-01] } # Change plans in the middle of the current plan period # and credit the balance of the current plan to the new plan iex&gt; Money.Subscription . change_plan current_plan , new_plan , current_interval_started : ~D[2018-03-01] , effective : ~D[2018-03-15] % Money.Subscription.Change { carry_forward : # Money &lt; :USD , 0 &gt; , credit_amount : # Money &lt; :USD , 5.49 &gt; , credit_amount_applied : # Money &lt; :USD , 5.49 &gt; , credit_days_applied : 0 , credit_period_ends : nil , first_billing_amount : # Money &lt; :USD , 4.51 &gt; , first_interval_starts : ~D[2018-03-15] , next_interval_starts : ~D[2018-06-15] } # Change plans in the middle of the current plan period # but instead of a monetary credit, apply the credit as # extra days on the new plan in the first billing period iex&gt; Money.Subscription . change_plan current_plan , new_plan , current_interval_started : ~D[2018-03-01] , effective : ~D[2018-03-15] , prorate : :period % Money.Subscription.Change { carry_forward : # Money &lt; :USD , 0 &gt; , credit_amount : # Money &lt; :USD , 5.49 &gt; , credit_amount_applied : # Money &lt; :USD , 0 &gt; , credit_days_applied : 51 , credit_period_ends : ~D[2018-05-04] , first_billing_amount : # Money &lt; :USD , 10 &gt; , first_interval_starts : ~D[2018-03-15] , next_interval_starts : ~D[2018-08-05] } # Create a subscription iex&gt; plan = Money.Subscription.Plan . new! ( Money . new ( :USD , 200 ) , :month , 3 ) iex&gt; subscription = Money.Subscription . new! plan , ~D[2018-01-01] % Money.Subscription { created_at : # DateTime &lt; 2018 - 03 - 23 07 : 45 : 44.418916 Z &gt; , id : nil , plans : [ { % Money.Subscription.Change { carry_forward : # Money &lt; :USD , 0 &gt; , credit_amount : # Money &lt; :USD , 0 &gt; , credit_amount_applied : # Money &lt; :USD , 0 &gt; , credit_days_applied : 0 , credit_period_ends : nil , first_billing_amount : # Money &lt; :USD , 200 &gt; , first_interval_starts : ~D[2018-01-01] , next_interval_starts : ~D[2018-04-01] } , % Money.Subscription.Plan { interval : :month , interval_count : 3 , price : # Money &lt; :USD , 200 &gt; } } ] } # Change a subscription&#39;s plan iex&gt; new_plan = Money.Subscription.Plan . new! ( Money . new ( :USD , 150 ) , :day , 30 ) iex&gt; Money.Subscription . change_plan! subscription , new_plan % Money.Subscription { created_at : # DateTime &lt; 2018 - 03 - 23 07 : 47 : 48.593973 Z &gt; , id : nil , plans : [ { % Money.Subscription.Change { carry_forward : # Money &lt; :USD , 0 &gt; , credit_amount : # Money &lt; :USD , 0 &gt; , credit_amount_applied : # Money &lt; :USD , 0 &gt; , credit_days_applied : 0 , credit_period_ends : nil , first_billing_amount : # Money &lt; :USD , 150 &gt; , first_interval_starts : ~D[2018-04-01] , next_interval_starts : ~D[2018-05-01] } , % Money.Subscription.Plan { interval : :day , interval_count : 30 , price : # Money &lt; :USD , 150 &gt; } } , { % Money.Subscription.Change { carry_forward : # Money &lt; :USD , 0 &gt; , credit_amount : # Money &lt; :USD , 0 &gt; , credit_amount_applied : # Money &lt; :USD , 0 &gt; , credit_days_applied : 0 , credit_period_ends : nil , first_billing_amount : # Money &lt; :USD , 200 &gt; , first_interval_starts : ~D[2018-01-01] , next_interval_starts : ~D[2018-04-01] } , % Money.Subscription.Plan { interval : :month , interval_count : 3 , price : # Money &lt; :USD , 200 &gt; } } ] }","ref":"readme.html#subscriptions","title":"Introduction to Money - Subscriptions","type":"extras"},{"doc":"The companion package ex_money_sql provides functions for the serialization of Money data. See the README for further information.","ref":"readme.html#serializing-to-a-database-with-ecto","title":"Introduction to Money - Serializing to a database with Ecto","type":"extras"},{"doc":"Money can be installed by adding ex_money to your list of dependencies in mix.exs and then executing mix deps.get def deps do [ { :ex_money , &quot;~&gt; 5.0&quot; } , ... ] end","ref":"readme.html#installation","title":"Introduction to Money - Installation","type":"extras"},{"doc":"Fully localized formatting and rounding using ex_cldr Provides serialization to Postgres using a composite type and MySQL using a JSON type that keeps both the currency code and the amount together removing a source of potential error Uses the Decimal type in Elixir and the Postgres numeric type to preserve precision. For MySQL the amount is serialised as a string to preserve precision that might otherwise be lost if stored as a JSON numeric type (which is either an integer or a float) Includes a set of financial calculations (arithmetic and cash flow calculations) that follow solid rounding rules","ref":"readme.html#why-yet-another-money-package","title":"Introduction to Money - Why yet another Money package?","type":"extras"},{"doc":"The github gist gives a good summary of the challenges of managing money in an application. The following described how Money handles each of these assertions. 1. You can store a price in a floating point variable. Money operates and serialises in a arbitrary precision Decimal value. 2. All currencies are subdivided in 1/100th units (like US dollar/cents, euro/eurocents etc.). Money leverages CLDR which defines the appropriate number of decimal places of a currency. As of CLDR version 32 there are: 52 currencies with zero decimal digits 241 currencies with two decimal digits 6 currencies with three decimal digits and 1 currency with four decimal digits 3. All currencies are subdivided in decimal units (like dinar/fils) 4. All currencies currently in circulation are subdivided in decimal units. (to exclude shillings, pennies) (counter-example: MGA) 5. All currencies are subdivided. (counter-examples: KRW, COP, JPY... Or subdivisions can be deprecated.) Money correctly manages the appropriate number of decimal places for a currency. It also round correctly when formatting a currency for output (different currencies have different rounding levels for cash or transactions). 6. Prices can't have more precision than the smaller sub-unit of the currency. (e.g. gas prices) All Money calculations are done with decimal arithmetic to the maxium precision of 28 decimal digits. 7. For any currency you can have a price of 1. (ZWL) Money makes no assumption about the value assigned as long as its a real number. 8. Every country has its own currency. (EUR is the best example, but also Franc CFA, etc.) Money makes no assumptions about the linkage of currencies to territories. 9. No country uses another's country official currency as its official currency. (many countries use USD: Ecuador, Micronesia...) 10. Countries have only one currency. Money doesn't link territories (countries) to a currency - it focuses only on the Money domain. The addon package cldr_territories does have knowledge of what curriencies are in effect throughout history for a given territory. See Cldr.Territory.info/1 . 11. Countries have only one currency currently in circulation. (Panama officially uses both PAB and USD) Money makes no assumptions about currencies in circulation. 12. I'll only deal with currencies currently in circulation anyway. Money makes no assumptions about currencies in circulation. 13. All currencies have an ISO 4217 3-letter code. (The Transnistrian ruble has none, for example) Money does validate currency codes against the ISO 4217 list. Custom currencies can be created in accordance with ISO 4217 using Cldr.Currency.new/2 . 14. All currencies have a different name. (French franc, &quot;nouveau franc&quot;) Money has localised names for all ISO 4217 currencies in each of the over 500 locales defined by CLDR. 15. You always put the currency symbol after the price. Money formats currency strings according to a format mask that is either defined by CLDR or user supplied. 16. You always put the currency symbol before the price. Money formats currency strings according to a format mask that is either defined by CLDR or user supplied. 17. You always put the currency symbol either after, or before the price, never in the middle. Money formats currency strings according to a format mask that is either defined by CLDR or user supplied. 18. There's only one currency symbol for any currency. (元, 角, 分 are increasing units of the Chinese renminbi.) Money uses format masks defined by CLDR which, for the Chinese renminbi uses the &quot;￥&quot; symbol. 19. For a given currency, you always, but always, put the symbol in the same place. Money makes no assumpions about symbol placement. The symbol can be places anywhere in a formatted string but is typically, for CLDR format masks, placed either before or after the formatted number. 20. OK. But if you only use the ISO 4217 currency codes, you always put it before the price. (Hint: it depends on the language.) Same as for the answer to 19 above. 21. Before the price means on the left. (ILS) Money formats according to a locale and correctly places symbols for languages written right-to-left. 22. You can always use a dot (or a comma, etc.) as a decimal separator. The decimal separator is defined per locale according to the CLDR definitions. 23. You can always use a space (or a dot, or a comma, etc.) as a thousands separator. The thousands (acutally grouping since not all locales format in thousands) separator is defined per locale according to the CLDR definitions. 24. You separate big prices by grouping numbers in triplets (thousands). (One writes ¥1 0000) Grouping is done according the CLDR definitions. For many languages the grouping is in thousands. Some format other ways. For example in India numbers are formatted with the first group as a triplet and subsequent groups as doublets. 25. Prices at a single company will never range from five digits before the decimal to five digits after. Money 's default precision is 28 decimal digits. All arithmetic is done using arbitrary precision decimal arithemetic. No round is performed unless either explicitly requested or a money value is formatted for output. When formatting rounding is applied according the locale-specific rules. 26. Prices contains only digits and punctuation. (Germans can write 12,- €) Money format masks can contain very flexible formatting masks. A set of formats is defined for each locale and a user-defined masks can also be defined. 27. A price can be at most 10^N for some value of N. See the answer to 25. 28. Given two currencies, there is only one exchange rate between them at any given point in time. Money supports an exchange rate mechansim, currency conversions and retrieval from external exchange rate services. It does not impose any constraint on underlying conversion tables. 29. Given two currencies, there is at least one exchange rate between them at any given point in time. (restriction on export of MAD, ARS, CNY, for example) See the answer to 28. 30. And the final one: a standalone $ character is always pronounced dollar. (It's also the peso sign.) This is outside the domain of Money .","ref":"readme.html#falsehoods-programmers-believe-about-prices","title":"Introduction to Money - Falsehoods programmers believe about prices","type":"extras"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"This is the changelog for Money v5.13.0 released on April 28th, 2023. For older changelogs please consult the release tag on GitHub Note ex_money 5.13.0 is supported on Elixir 1.11 and later versions only. It also requires ex_cldr_numbers 2.31 or later. Enhancements Updates to ex_cldr version 2.37.0 which includes data from CLDR release 43 Tests now assume Decimal ~&gt; 2.0 since the Inspect protocol implementation now emits executable code examples.","ref":"changelog.html#money-v5-13-0","title":"Changelog - Money v5.13.0","type":"extras"},{"doc":"This is the changelog for Money v5.12.4 released on March 30th, 2023. For older changelogs please consult the release tag on GitHub Note ex_money 5.12.4 is supported on Elixir 1.10 and later versions only. It also requires ex_cldr_numbers 2.25 or later. Bug Fixes Delegates http requests (used in exchange rates retrieval) to Cldr.Http.get_with_headers/2 . This centralizes all HTTP get requests for all ex_cldr_* libraries to this one function which can then be reviewed and managed for security concerns.","ref":"changelog.html#money-v5-12-4","title":"Changelog - Money v5.12.4","type":"extras"},{"doc":"This is the changelog for Money v5.12.3 released on October 13th, 2022. For older changelogs please consult the release tag on GitHub Note ex_money 5.12.3 is supported on Elixir 1.10 and later versions only. It also requires ex_cldr_numbers 2.25 or later. Bug Fixes Fix NaN and Inf amount detection to be compatible with Decimal 1.x and 2.x. Thanks to @Lostkobrakai for the PR. Closes #144.","ref":"changelog.html#money-v5-12-3","title":"Changelog - Money v5.12.3","type":"extras"},{"doc":"This is the changelog for Money v5.12.2 released on October 13th, 2022. For older changelogs please consult the release tag on GitHub Note ex_money 5.12.2 is supported on Elixir 1.10 and later versions only. It also requires ex_cldr_numbers 2.25 or later. Bug Fixes Don't create &quot;NaN&quot; or &quot;Inf&quot; valued Money structs. Thanks for @coladarci for the report. Closes #143.","ref":"changelog.html#money-v5-12-2","title":"Changelog - Money v5.12.2","type":"extras"},{"doc":"This is the changelog for Money v5.12.1 released on August 27th, 2022. For older changelogs please consult the release tag on GitHub Note ex_money 5.12.1 is supported on Elixir 1.10 and later versions only. It also requires ex_cldr_numbers 2.25 or later. Bug Fixes Removes compile-time warnings for Elixir 1.14 (use Application.compile_env/2 , not Application.get_env/2 )","ref":"changelog.html#money-v5-12-1","title":"Changelog - Money v5.12.1","type":"extras"},{"doc":"This is the changelog for Money v5.12.0 released on June 8th, 2022. For older changelogs please consult the release tag on GitHub Note ex_money 5.12.0 is supported on Elixir 1.10 and later versions only. It also requires ex_cldr_numbers 2.25 or later. Enhancements Add Money.localize/2 to convert a money amount into the currency in affect for the given locale.","ref":"changelog.html#money-v5-12-0","title":"Changelog - Money v5.12.0","type":"extras"},{"doc":"This is the changelog for Money v5.11.0 released on May 14th, 2022. For older changelogs please consult the release tag on GitHub Note ex_money 5.11.0 is supported on Elixir 1.10 and later versions only. It also requires ex_cldr_numbers 2.25 or later. Enhancements Adds support for ISO 24165 Digital Tokens (crypto currency) . Digital Token-based money behaves the same as currency-based money with the following exceptions due to limited data availability: Digital token names are not localized (there is no localised data available in CLDR) Digital token names are not pluralized (also because there is no localised data available) Digital token amounts are never rounded (there is no data available to standardise on rounding rules or the number of fractional digits to round to)","ref":"changelog.html#money-v5-11-0","title":"Changelog - Money v5.11.0","type":"extras"},{"doc":"This is the changelog for Money v5.10.0 released on April 6th, 2022. For older changelogs please consult the release tag on GitHub Note ex_money 5.10.0 is supported on Elixir 1.10 and later versions only. It also requires ex_cldr_numbers 2.25 or later. Enhancements Add Money.zero?/1 , Money.positive?/1 and Money.negative?/1 . Thanks to @emaiax for the PR. Update CLDR to release 41 in ex_cldr version 2.28.0 and ex_cldr_numbers 2.26.0 .","ref":"changelog.html#money-v5-10-0","title":"Changelog - Money v5.10.0","type":"extras"},{"doc":"This is the changelog for Money v5.9.0 released on February 21st, 2022. For older changelogs please consult the release tag on GitHub Note ex_money 5.9.0 is supported on Elixir 1.10 and later versions only. It also requires ex_cldr_numbers 2.25 or later. Enhancements Updates to ex_cldr version 2.26.0 and ex_cldr_numbers version 2.25.0 which use atoms for locale names and rbnf locale names. This is consistent with other elements of t:Cldr.LanguageTag where atoms are used when the cardinality of the data is fixed and relatively small and strings where the data is free format. Adjusts the output of Money.inspect/2 to be executable code. Instead of #Money&lt;:USD, 100&gt; the output will be Money.new(:USD, &quot;100&quot;) . This improved developer experience by allowing for copy/paste of inspect/2 results into iex . It is also in line with similar changes being made in elixir , Decimal and others. Add documentation for :currency_symbol option for Money.to_string/2 . Although its an option that is passed through to Cldr.Number.to_string/3 , its very relevant to t:Money formatting.","ref":"changelog.html#money-v5-9-0","title":"Changelog - Money v5.9.0","type":"extras"},{"doc":"This is the changelog for Money v5.8.0 released on January 31st, 2022. For older changelogs please consult the release tag on GitHub Note ex_money 5.8.0 is supported on Elixir 1.10 and later versions only. It also requires ex_cldr_numbers 2.23 or later. Enhancements Adds configuration option :exclude_protocol_implementations to omit generating implementations for one or more protocols from the list Json.Encoder , Phoneix.HTML.Safe and Gringotts.Money . Thanks to @jgough-playoxygen for the suggestion .","ref":"changelog.html#money-v5-8-0","title":"Changelog - Money v5.8.0","type":"extras"},{"doc":"This is the changelog for Money v5.7.4 released on December 23rd, 2021. For older changelogs please consult the release tag on GitHub Note ex_money 5.7.4 is supported on Elixir 1.10 and later versions only. It also requires ex_cldr_numbers 2.23 or later. Bug Fixes Fix Money.to_integer_exp/1 when t:Money has a negative amount. Thanks to @hamptokr for the report and the PR.","ref":"changelog.html#money-v5-7-4","title":"Changelog - Money v5.7.4","type":"extras"},{"doc":"This is the changelog for Money v5.7.3 released on December 19th, 2021. For older changelogs please consult the release tag on GitHub Note ex_money 5.7.3 is supported on Elixir 1.10 and later versions only. It also requires ex_cldr_numbers 2.23 or later. Bug Fixes Fixes retrieving exchange rates on OTP releases before OTP 22. Thanks to @fbettag for the report, collaboration and patience.","ref":"changelog.html#money-v5-7-3","title":"Changelog - Money v5.7.3","type":"extras"},{"doc":"This is the changelog for Money v5.7.2 released on December 17th, 2021. For older changelogs please consult the release tag on GitHub Note ex_money 5.7.2 is supported on Elixir 1.10 and later versions only. It also requires ex_cldr_numbers 2.23 or later. Bug Fixes Fix spec for Money.from_integer/3 . Thanks to @jdewar for the report. Enhancements Support a :fractional_digits option for Money.from_integer/3 and improve the documentation.","ref":"changelog.html#money-v5-7-2","title":"Changelog - Money v5.7.2","type":"extras"},{"doc":"This is the changelog for Money v5.7.1 released on December 8th, 2021. For older changelogs please consult the release tag on GitHub Note ex_money 5.7.1 is supported on Elixir 1.10 and later versions only. It also requires ex_cldr_numbers 2.23 or later. Bug Fixes Fix dialyzer warnings on Elixir 1.12 and 1.13 Replace use Mix.Config with import Config in configuration files since the former is deprecated.","ref":"changelog.html#money-v5-7-1","title":"Changelog - Money v5.7.1","type":"extras"},{"doc":"This is the changelog for Money v5.7.0 released on October 28th, 2021. For older changelogs please consult the release tag on GitHub Enhancements Updates to support CLDR release 40 via ex_cldr version 2.24 Deprecations Don't call deprecated Cldr.Config.get_locale/2 , use Cldr.Locale.Loader.get_locale/2 instead.","ref":"changelog.html#money-v5-7-0","title":"Changelog - Money v5.7.0","type":"extras"},{"doc":"This is the changelog for Money v5.6.0 released on August 31st, 2021. For older changelogs please consult the release tag on GitHub Enhancements Adds Money.to_currency_code/1 to return the currency code part of a t:Money . Thanks to @Adzz for the proposal. Closes #130.","ref":"changelog.html#money-v5-6-0","title":"Changelog - Money v5.6.0","type":"extras"},{"doc":"This is the changelog for Money v5.5.5 released on August 15th, 2021. For older changelogs please consult the release tag on GitHub Bug Fixes Allow either phoenix_html version 2.x or 3.x. Thanks to @seantanly for the PR. Closes #129.","ref":"changelog.html#money-v5-5-5","title":"Changelog - Money v5.5.5","type":"extras"},{"doc":"This is the changelog for Money v5.5.4 released on June 17th, 2021. For older changelogs please consult the release tag on GitHub Bug Fixes Support t:Cldr.Number.Format.Options as an argument to Money.to_string/2 . Thanks to @jeroenvisser101 for the PR. Closes #127.","ref":"changelog.html#money-v5-5-4","title":"Changelog - Money v5.5.4","type":"extras"},{"doc":"This is the changelog for Money v5.5.3 released on May 7th, 2021. For older changelogs please consult the release tag on GitHub Bug Fixes Fixes parsing money when a currency string has a &quot;.&quot; in it such as &quot;kr.&quot;. Thanks for the report to @Doerge. Closes #125.","ref":"changelog.html#money-v5-5-3","title":"Changelog - Money v5.5.3","type":"extras"},{"doc":"This is the changelog for Money v5.5.2 released on April 14th, 2021. For older changelogs please consult the release tag on GitHub Bug Fixes Fix exception message when describing the requirement for a default backend configuration. Thanks to @holandes22 for the report. Closes #124.","ref":"changelog.html#money-v5-5-2","title":"Changelog - Money v5.5.2","type":"extras"},{"doc":"This is the changelog for Money v5.5.1 released on February 18th, 2021. For older changelogs please consult the release tag on GitHub Bug Fixes Fix formatting a t:Money that has no :format_options key. That can happen if re-hydrating a t:Money using :erlang.binary_to_term/1 from an older version of ex_money that doesn't have the :format_options key in the struct. Thanks to @coladarci. Fixes #123.","ref":"changelog.html#money-v5-5-1","title":"Changelog - Money v5.5.1","type":"extras"},{"doc":"This is the changelog for Money v5.5.0 released on February 10th, 2021. For older changelogs please consult the release tag on GitHub Enhancements Adds format options to t:Money to allow per-money formatting options to be applied with the String.Chars protocol. Thanks to @morinap for the feature request. Adds Money.put_format_options/2","ref":"changelog.html#money-v5-5-0","title":"Changelog - Money v5.5.0","type":"extras"},{"doc":"This is the changelog for Money v5.4.1 released on January 7th, 2021. For older changelogs please consult the release tag on GitHub Bug Fixes Update stream_data to remove stacktrace warning Use Cldr.default_backend!/0 instead of deprecated Cldr.default_backend/0 in tests. Closes #120. Thanks to @darwintantuco.","ref":"changelog.html#money-v5-4-1","title":"Changelog - Money v5.4.1","type":"extras"},{"doc":"This is the changelog for Money v5.4.0 released on November 1st, 2020. For older changelogs please consult the release tag on GitHub Enhancements Add support for CLDR 38","ref":"changelog.html#money-v5-4-0","title":"Changelog - Money v5.4.0","type":"extras"},{"doc":"This is the changelog for Money v5.3.2 released on September 30th, 2020. For older changelogs please consult the release tag on GitHub Bug Fixes Fix docs for Money.div!/2 Update cldr_utils which implements a shim for Decimal to support both version 1.9 and 2.0 .","ref":"changelog.html#money-v5-3-2","title":"Changelog - Money v5.3.2","type":"extras"},{"doc":"This is the changelog for Money v5.3.1 released on September 26th, 2020. For older changelogs please consult the release tag on GitHub Bug Fixes Support nimble_parsec versions that match ~&gt; 0.5 or ~&gt; 1.0","ref":"changelog.html#money-v5-3-1","title":"Changelog - Money v5.3.1","type":"extras"},{"doc":"This is the changelog for Money v5.3.0 released on September 5th, 2020. For older changelogs please consult the release tag on GitHub Bug Fixes Fix parsing money amounts to use Unicode definition of whitespace (set [:Zs:] ). Thanks to @Sanjibukai for the report. Enhancements Add Money.sum/2 to sum a list of Money , converting them if required.","ref":"changelog.html#money-v5-3-0","title":"Changelog - Money v5.3.0","type":"extras"},{"doc":"This is the changelog for Money v5.2.1 released on June 23rd, 2020. For older changelogs please consult the release tag on GitHub Enhancements Configure the Money.Application supervisor via the arguments to Money.Application.start/2 and configure defaults in mix.exs . This permits different restart strategies and names. Add Money.ExchangeRates.Supervisor.default_supervisor/0 to return the name of the default supervisor which is Money.Supervisor Change Money.ExchangeRates.Supervisor.stop/0 to become Money.ExchangeRates.Supervisor.stop/{0, 1} allowing the supervisor name to be passed in. The default is Money.ExchangeRates.Supervisor.default_supervisor/0 Bug Fixes Add back the name of the Application supervisor, Money.Supervisor . Thanks for the report of the regression to @jeroenvisser101. Fixes #117.","ref":"changelog.html#money-v5-2-1","title":"Changelog - Money v5.2.1","type":"extras"},{"doc":"This is the changelog for Money v5.2.0 released on May 30th, 2020. For older changelogs please consult the release tag on GitHub Enhancements Adds a configuration option :verify_peer which is a boolean that determines whether to verify the client certificate for any exchange rate service API call. The default is true . This option should not be changed without a very clear understanding of the security implications. This option will remain undocumented but supported for now. Bug fixes Handle expired certificate errors on the exchange rates API service and log them. Thanks to @coladarci. Fixes #116","ref":"changelog.html#money-v5-2-0","title":"Changelog - Money v5.2.0","type":"extras"},{"doc":"This is the changelog for Money v5.1.0 released on May 26th, 2020. For older changelogs please consult the release tag on GitHub Enhancements Extract default currency from locale when calling Money.parse/2 on a money string. The updated docs now say: :default_currency is any valid currency code or false that will used if no currency code, symbol or description is indentified in the parsed string. The default is nil which means that the default currency associated with the :locale option will be used. If false then the currency assocated with the :locale option will not be used and an error will be returned if there is no currency in the string being parsed. Add certificate verification for exchange rate retrieval","ref":"changelog.html#money-v5-1-0","title":"Changelog - Money v5.1.0","type":"extras"},{"doc":"This is the changelog for Money v5.0.2 released on April 29th, 2020. For older changelogs please consult the release tag on GitHub Bug Fixes Update the application supervisor spec","ref":"changelog.html#money-v5-0-2","title":"Changelog - Money v5.0.2","type":"extras"},{"doc":"This is the changelog for Money v5.0.1 released on January 28th, 2020. For older changelogs please consult the release tag on GitHub Bug Fixes Make nimble_parsec a required dependency since it is required for parsing money amounts. Thanks to @jonnystoten for the report.","ref":"changelog.html#money-v5-0-1","title":"Changelog - Money v5.0.1","type":"extras"},{"doc":"This is the changelog for Money v5.0.0 released on January 21st, 2020. For older changelogs please consult the release tag on GitHub Breaking changes Elixir 1.10 introduces semantic sorting for stucts that depends on the availability of a compare/2 function that returns :lt , :eq or :gt . Therefore in this release of ex_money the functions compare/2 and compare!/2 are swapped with cmp/2 and cmp!/2 in order to conform with this expectation. Now compare/2 will return :eq , :lt or :gt . And cmp/2 return -1 , 0 or 1 . Deprecate Money.reduce/1 in favour of Money.normalize/1 to be consistent with Decimal versions 1.9 and later. It is believed and tested that Money version 5.0.0 is compatible with all versions of Decimal from 1.6 up to the as-yet-unreleased 2.0 . Support of Elixir 1.10 Enum sorting From Elixir verison 1.10.0 , several functions in the Enum module can use the Money.compare/2 function to simplify sorting. For example: iex&gt; list = [ Money . new ( :USD , 100 ) , Money . new ( :USD , 200 ) ] [ # Money &lt; :USD , 100 &gt; , # Money &lt; :USD , 200 &gt; ] iex&gt; Enum . sort list , Money [ # Money &lt; :USD , 100 &gt; , # Money &lt; :USD , 200 &gt; ] iex&gt; Enum . sort list , { :asc , Money } [ # Money &lt; :USD , 100 &gt; , # Money &lt; :USD , 200 &gt; ] iex&gt; Enum . sort list , { :desc , Money } [ # Money &lt; :USD , 200 &gt; , # Money &lt; :USD , 100 &gt; ] Note that Enum.sort/2 will sort money amounts even when the currencies are incompatible. In this case the order of the result is not predictable. It is the developers responsibility to filter the list to compatible currencies prior to sorting. This is a limitation of the Enum.sort/2 implementation. Notes on Decimal version support ex_money version 5.0.0 is compatible with Decimal versions from 1.6 onwards. In Decimal version 2.0 the same changes to compare/2 and cmp/2 will occur and in Decimal version 1.9 , Decimal.cmp/2 is deprecated. ex_money version 5.0.0 detects these different versions of Decimal and therefore remains compatability with Decimal back to version 1.6 .","ref":"changelog.html#money-v5-0-0","title":"Changelog - Money v5.0.0","type":"extras"},{"doc":"This is the changelog for Money v4.4.2 released on January 2nd, 2020. For older changelogs please consult the release tag on GitHub Bug Fixes Remove calls to Code.ensure_compiled?/1 since it is deprecated in Elixir 1.10. Use instead Cldr.Config.ensure_compiled?/1 which is added as a private API in Cldr version 2.12.0. This version of Cldr now becomes the minimum version required. Remove spurious entries in .dialyzer_ignore_warnings - no entries are required and dialyzer is happy.","ref":"changelog.html#money-v4-4-2","title":"Changelog - Money v4.4.2","type":"extras"},{"doc":"This is the changelog for Money v4.4.1 released on November 10th, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Fixes money parsing error. Thanks to @Doerge. Closes #112.","ref":"changelog.html#money-v4-4-1","title":"Changelog - Money v4.4.1","type":"extras"},{"doc":"This is the changelog for Money v4.4.0 released on November 6th, 2019. For older changelogs please consult the release tag on GitHub Breaking Change Money.parse/2 until this release supported the :currency_filter option. It allowed for currencies to be filtered based upon their attributes ( :all , :current , :historic , :tender , :annotated ). When multiple attributes were passed in a list, a currency had to meet all of these attributes. From this release onwards, multiple attributes items are or ed, not and ed. It is expected this option is used extremely rarely and therefore of limited impact. Enhancements Money.parse/2 now includes the option :default_currency which allows for parsing a number only (without a currency code) and it will be tagged with the :default_currency . iex&gt; Money . parse ( &quot;100&quot; ) { :error , { Money.Invalid , &quot;A currency code, symbol or description must be specified but was not found in \\&quot; 100 \\&quot; &quot; } } iex&gt; Money . parse ( &quot;100&quot; , default_currency : :USD ) # Money &lt; :USD , 100 &gt; Add :only and :except options to Money.parse/2 to specify which currency codes or currency attributes are permitted. :only and :except replace the option :currency_filter which is now deprecated. If provided, :currency_filter is interpreted as :only . An example: iex&gt; Money . parse ( &quot;100 usd&quot; , only : :current , except : :USD ) { :error , { Money.UnknownCurrencyError , &quot;The currency \\&quot; usd \\&quot; is unknown or not supported&quot; } } Money.parse/2 now supports negative money amounts. iex&gt; Money . parse ( &quot;chf -100&quot; ) # Money &lt; :CHF , - 100 &gt; iex&gt; Money . parse ( &quot;(chf 100)&quot; ) # Money &lt; :CHF , - 100 &gt; The money parser has been rewritten using nimble_parsec","ref":"changelog.html#money-v4-4-0","title":"Changelog - Money v4.4.0","type":"extras"},{"doc":"This is the changelog for Money v4.3.0 released on September 8th, 2019. For older changelogs please consult the release tag on GitHub Enhancements Adds a Money backend in the same spirit as other libraries that leverge ex_cldr . Thanks to @Lostkobrakai. Closes #108. All of the functions in the Money module may also be called on a backend module &lt;backend&gt;.Money.fun without having to specify a backend module since this is implicit. Bug Fixes Money.new!/3 replaces Money.new!/2 to accept options. Thanks to @Lostkobrakai. Closes #109.","ref":"changelog.html#money-v4-3-0","title":"Changelog - Money v4.3.0","type":"extras"},{"doc":"This is the changelog for Money v4.2.2 released on September 7th, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Use Keyword.get_lazy when the default is Cldr.default_backend/0 to avoid exceptions when no default backend is configured. Thanks to @Lostkobrakai. Closes #108.","ref":"changelog.html#money-v4-2-2","title":"Changelog - Money v4.2.2","type":"extras"},{"doc":"This is the changelog for Money v4.2.1 released on September 2nd, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Fixes parsing of money amount that have a single digit amount. Closes #107. Thanks to @njwest","ref":"changelog.html#money-v4-2-1","title":"Changelog - Money v4.2.1","type":"extras"},{"doc":"This is the changelog for Money v4.2.0 released on 21 August, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Move the Money.Migration module to ex_money_ecto where it belongs Enhancements Money.default_backend/0 will now either use the backend configured under the :default_cldr_backend key of ex_money or Cldr.default_backend/0 . In either case an exeption will be raised if no default backend is configured.","ref":"changelog.html#money-v4-2-0","title":"Changelog - Money v4.2.0","type":"extras"},{"doc":"This is the changelog for Money v4.1.0 released on July 13th, 2019. For older changelogs please consult the release tag on GitHub Enhancements Adds Money.abs/1 . Thanks to @jeremyjh. Improve @doc consistency using ## Arguments not ## Options .","ref":"changelog.html#money-v4-1-0","title":"Changelog - Money v4.1.0","type":"extras"},{"doc":"This is the changelog for Money v4.0.0 released on July 8th, 2019. For older changelogs please consult the release tag on GitHub Breaking Changes Functions related to the serialization of money types have been extracted to the library ex_money_sql . For applications using the dependency ex_money that do not require serialization no changes are required. For applications using serialization, the dependency should be changed to ex_money_sql (which in turn depends on ex_money ). Supports Elixir 1.6 and later only","ref":"changelog.html#money-v4-0-0","title":"Changelog - Money v4.0.0","type":"extras"},{"doc":"This is the changelog for Money v3.4.4 released on June 2nd, 2019. For older changelogs please consult the release tag on GitHub Enhancements Supports passing an Cldr.Number.Formation.Options.t as alternative to a Keyword.t for options to Money.to_string/2 . Performance is doubled when using pre-validated options which is useful if formatting is being executed in a tight loop. An example of this usage is: iex&gt; money = Money . new ( :USD , 100 ) # Apply any options required as a keyword list # Money will take care of managing the `:currency` option iex&gt; options = [ ] iex&gt; { :ok , options } = Cldr.Number.Format.Options . validate_options ( 0 , backend , options ) iex&gt; Money . to_string ( money , options ) The 0 in validate_options is used to determine the sign of the amount because that can influence formatting - for example the accounting format often uses (1234) as its format. If you know your amounts are always positive, just use 0 . If the use case may have both positive and negative amounts, generate two option sets (one with the positive number and one with the negative). Then use the appropriate option set. For example: iex&gt; money = Money . new ( :USD , 1234 ) # Add options as required # Money will take care of managing the `:currency` option iex&gt; options = [ ] iex&gt; { :ok , positive_options } = Cldr.Number.Format.Options . validate_options ( 0 , backend , options ) iex&gt; { :ok , negative_options } = Cldr.Number.Format.Options . validate_options ( - 1 , backend , options ) iex&gt; if Money . cmp ( money , Money . zero ( :USD ) ) == :gt do ...&gt; Money . to_string ( money , positive_options ) ...&gt; else ...&gt; Money . to_string ( money , negative_options ) ...&gt; end","ref":"changelog.html#money-v3-4-4","title":"Changelog - Money v3.4.4","type":"extras"},{"doc":"This is the changelog for Money v3.4.3 released on June 2nd, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Ensure Money.to_string!/2 properly raises Add specs for Money.to_string/2 and Money.to_string!/2 Thanks to @rodrigues for the report and PR.","ref":"changelog.html#money-v3-4-3","title":"Changelog - Money v3.4.3","type":"extras"},{"doc":"This is the changelog for Money v3.4.2 released on April 16th, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Money.put_fraction/2 now correctly allows setting the fraction to 0. Enhancements Money.round/2 allows setting :currency_digits to an integer number of digits in addition to the options :iso , :cash and :accounting . The default remains :iso . Improves the documentation for Money.to_string/2 .","ref":"changelog.html#money-v3-4-2","title":"Changelog - Money v3.4.2","type":"extras"},{"doc":"This is the changelog for Money v3.4.1 released on April 5th, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Fix README.md markdown formatting error. Thanks to @fireproofsocks for the report and @lostkobrakai for the fix. Closes #99.","ref":"changelog.html#money-v3-4-1","title":"Changelog - Money v3.4.1","type":"extras"},{"doc":"This is the changelog for Money v3.4.0 released on March 28th, 2019. For older changelogs please consult the release tag on GitHub Enhancements Updates to CLDR version 35.0.0 released on March 27th 2019 through ex_cldr version 2.6.0.","ref":"changelog.html#money-v3-4-0","title":"Changelog - Money v3.4.0","type":"extras"},{"doc":"This is the changelog for Money v3.3.1 released on March 8th, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Fix or silence dialyzer warnings","ref":"changelog.html#money-v3-3-1","title":"Changelog - Money v3.3.1","type":"extras"},{"doc":"This is the changelog for Money v3.3.0 released on February 24th, 2019. For older changelogs please consult the release tag on GitHub Enhancements Adds Money.put_fraction/2 . This will set the fractional part of a money to the specified integer amount. Examples: iex&gt; Money . put_fraction Money . new ( :USD , &quot;2.49&quot; ) , 99 # Money &lt; :USD , 2.99 &gt; iex&gt; Money . put_fraction Money . new ( :USD , &quot;2.49&quot; ) , 999 { :error , { Money.InvalidAmountError , &quot;Rounding up to 999 is invalid for currency :USD&quot; } } Bug Fixes Parsing money strings now uses a more complete set of character definitions for decimal and grouping separators based upon the characters.json file of the &quot;en&quot; locale.","ref":"changelog.html#money-v3-3-0","title":"Changelog - Money v3.3.0","type":"extras"},{"doc":"This is the changelog for Money v3.2.4 released on February 13th, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Updates to ex_cldr_currencies version 2.1.2 which correctly removes duplicate currency strings when the same string referred to different currency codes. See the changelog for further detail. Enhancements Adds a :fuzzy option to Money.parse/2 that uses String.jaro_distance/2 to help determine if the provided currency text can be resolved as a currency code. For example: iex&gt; Money . parse ( &quot;100 eurosports&quot; , fuzzy : 0.8 ) # Money &lt; :EUR , 100 &gt; iex&gt; Money . parse ( &quot;100 eurosports&quot; , fuzzy : 0.9 ) { :error , { Money.Invalid , &quot;Unable to create money from \\\\ &quot; eurosports \\\\ &quot; and \\\\ &quot; 100 \\\\ &quot;&quot; } }","ref":"changelog.html#money-v3-2-4","title":"Changelog - Money v3.2.4","type":"extras"},{"doc":"This is the changelog for Money v3.2.3 released on February 12th, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Correctly parse money strings with unicode currency symbols like &quot;€&quot;. Closes #95. Thanks to @crbelaus.","ref":"changelog.html#money-v3-2-3","title":"Changelog - Money v3.2.3","type":"extras"},{"doc":"This is the changelog for Money v3.2.2 released on February 10th, 2019. For older changelogs please consult the release tag on GitHub Enhancements Improves parsing of money strings. Parsing now uses various strings that CLDR knows about. Some examples: iex&gt; Money . parse &quot;$au 12 346&quot; , locale : &quot;fr&quot; # Money &lt; :AUD , 12346 &gt; iex&gt; Money . parse &quot;12 346 dollar australien&quot; , locale : &quot;fr&quot; # Money &lt; :AUD , 12346 &gt; iex&gt; Money . parse &quot;A$ 12346&quot; , locale : &quot;en&quot; # Money &lt; :AUD , 12346 &gt; iex&gt; Money . parse &quot;australian dollar 12346.45&quot; , locale : &quot;en&quot; # Money &lt; :AUD , 12346.45 &gt; iex&gt; Money . parse &quot;AU$ 12346,45&quot; , locale : &quot;de&quot; # Money &lt; :AUD , 12346.45 &gt; # Can also return the strings available for a given currency # and locale iex&gt; Cldr.Currency . strings_for_currency :AUD , &quot;de&quot; [ &quot;aud&quot; , &quot;au$&quot; , &quot;australischer dollar&quot; , &quot;australische dollar&quot; ] # Round trip formatting also seems to be ok iex&gt; { :ok , string } = Cldr.Number . to_string 1234 , Money.Cldr , currency : :AUD iex&gt; Money . parse string # Money &lt; :AUD , 1234.00 &gt;","ref":"changelog.html#money-v3-2-2","title":"Changelog - Money v3.2.2","type":"extras"},{"doc":"This is the changelog for Money v3.2.1 released on February 2nd, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Added Money.Ecto.Composite.Type.cast/1 and Money.Ecto.Map.Type.cast/1 for a String.t parameter. When a String.t is provided, cast/1 will call Money.parse/2 to create the Money.t . Money.new/3 now uses the current locale on the default backend if no locale or backend is specified. This means that Money.Ecto.Composite.Type.cast/1 and Money.Ecto.Map.Type.cast/1 will be parsed using the locale that has been set for the current process in the default backend. As a result, a simple type=text form field can be used to input a money type (currency code and amount in a single string) that can then be cast to a Money.t .","ref":"changelog.html#money-v3-2-1","title":"Changelog - Money v3.2.1","type":"extras"},{"doc":"This is the changelog for Money v3.2.0 released on February 1st, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Correctly generate migrations_path/1 function based upon whether Ecto is configured and which version Enhancements Adds Money.parse/2 which will parse a string comprising a currency code and an amount. It will return a Money.t or an error. This function may be helpful in supporting money input in HTML forms.","ref":"changelog.html#money-v3-2-0","title":"Changelog - Money v3.2.0","type":"extras"},{"doc":"This is the changelog for Money v3.1.0 released on December 30th, 2018. For older changelogs please consult the release tag on GitHub Bug Fixes Fix typo in exchange_rates_retriever.ex . Thanks to @lostkobrakai. Closes #91. Remove obsolete cldr compiler Changes the sum aggregate function for money_with_currency to be STRICT which means it handles NULL columns in the same way as the standard SUM function. Thanks to @lostkobrakai. Closes #88. Fixes documentation link errors Fix unhandled terminate typo error in exchange rates server. Thanks to @xavier. Closes #90.","ref":"changelog.html#money-v3-1-0","title":"Changelog - Money v3.1.0","type":"extras"},{"doc":"This is the changelog for Money v3.0.0 released on November 23rd, 2018. For older changelogs please consult the release tag on GitHub The primary purpose of this release is to support ex_cldr version 2.0 Breaking changes Money.from_tuple/1 has been removed Uses ex_cldr version 2. Please see the changelog for configuration changes that are required. Requires a default_cldr_backend to be configured in config.exs . For example: config :ex_money , ... default_cldr_backend : MyApp.Cldr end","ref":"changelog.html#money-v3-0-0","title":"Changelog - Money v3.0.0","type":"extras"},{"doc":"","ref":"license.html","title":"LICENSE","type":"extras"},{"doc":"Copyright 2017-2021 Kip Cole Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http :// www . apache . org / licenses / LICENSE - 2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"license.html#license","title":"LICENSE - License","type":"extras"}]